from unittest.mock import Mock
from unittest import mock
import idmap.tls
import django.db
from django.core.management import call_command
from otree.common import Currency
from otree.db import models
from otree.models import Participant, Session
from otree.models.varsmixin import ModelWithVars
from otree.db.idmap import (
    save_objects, _get_save_objects_model_instances, use_cache)
from .utils import TestCase, run_bots
from tests.simple.models import Player
from otree.db.serializedfields import VarsError
from otree.api import BaseConstants
import django.db.models
import numpy

class SavingTests(TestCase):
    # We need to make sure to flush the idmap cache here after every save in
    # order to prevent getting values that do not actually represent the DB
    # values.

    def test_dont_save_if_no_change(self):
        call_command('create_session', 'simple', '1')

        with use_cache():
            participant = Participant.objects.get()

            # We keep track of the participant.
            instances = _get_save_objects_model_instances()
            self.assertEqual(instances, [participant])

            # But we won't save the participant since we didn't change it.
            with self.assertNumQueries(0):
                save_objects()


    def test_save_only_changed_fields(self):
        call_command('create_session', 'simple', '1')
        with use_cache():
            participant = Participant.objects.get()
            with mock.patch.object(django.db.models.Model, 'save') as patched_save:
                save_objects()
                # in save-the-change 2017 version, save is not called at all
                patched_save.assert_called_once_with(update_fields=[])

                participant.code = 'hello'
                save_objects()
                # but is this necessarily the only argument? no
                patched_save.assert_called_with(update_fields=['code'])


    def test_nested_changes(self):
        call_command('create_session', 'simple', '1')
        # Reset cache.
        with use_cache():
            # Query participant via session.
            session = Session.objects.get()
            participant = session.participant_set.get()
            participant.is_on_wait_page = not participant.is_on_wait_page

            # Save participant.
            with self.assertNumQueries(1):
                save_objects()

    def test_with_app_models(self):
        call_command('create_session', 'simple', '2')

        from .simple.models import Player

        with use_cache():
            players = Player.objects.all()
            self.assertEqual(len(players), 2)

            group = players[0].group
            group.save = Mock()
            group.round_number += 1

            # Query session object to test that it's loaded..
            group.session
            participants = group.session.participant_set.all()

            all_instances = {
                players[0],
                players[1],
                group,
                group.session,
                participants[0],
                participants[1]
            }

            self.assertEqual(
                set(_get_save_objects_model_instances()),
                all_instances)

            # No queries are executed. The group model shall be saved, but we
            # mocked out the save method. All other models should be left
            # untouched.
            with self.assertNumQueries(0):
                save_objects()

            self.assertTrue(group.save.called)

class SavingBots(TestCase):
    def test_bots(self):
        run_bots('saving', 1)


class PickleFieldTests(TestCase):

    def test_pickle(self):
        self.test_values = [
            1, 1.33,
            None,
            [1, 2, 3., {}, ["coso", None, {}]],
            {"foo": [], "algo": None, "spam": [1, 2, 3.]},
            [],
            {},
            {1.5: 2},
            {Currency(1): Currency(2)},
            {1,2,3}
        ]
        for value in self.test_values:
            field = models._PickleField(default=value)
            serialized = field.get_prep_value(value)
            restored = field.to_python(serialized)
            self.assertEquals(value, restored)


class TestModelWithVars(ModelWithVars):
    pass


class Constants(BaseConstants):
    alist = [1,2]


class VarsTests(TestCase):

    def test_vars_are_saved(self):
        o = TestModelWithVars.objects.create()
        o.vars['a'] = 'b'
        o.save()

        o = TestModelWithVars.objects.get()
        self.assertEqual(o.vars, {'a': 'b'})

        o.vars['a'] = 'd'
        o.save()

        o = TestModelWithVars.objects.get()
        self.assertEqual(o.vars, {'a': 'd'})

    def test_forbid_storing_models(self):
        call_command('create_session', 'simple', '1')

        session = Session.objects.get()
        participant = Participant.objects.get()
        player = Player.objects.get()

        for value in [
            player,
            [player],
            {1: player},
            participant
        ]:
            session.vars['something_with_a_model'] = value
            with self.assertRaises(VarsError):
                session.save()

    def test_storing_constants(self):

        self.assertEqual(Constants.alist, [1,2])
        o = TestModelWithVars.objects.create()
        # users should always be using .copy(), even when assigning to vars,
        # because they might modify it before it's saved.
        o.vars['alist'] = Constants.alist.copy()
        o.save()

        o = TestModelWithVars.objects.get()
        vars_alist = o.vars['alist']
        self.assertEqual(vars_alist, Constants.alist)
        vars_alist.append(3)

        self.assertEqual(vars_alist, [1,2,3])
        # should not modify the original
        self.assertEqual(Constants.alist, [1,2])

    def test_numpy(self):
        '''
        https://github.com/karanlyons/django-save-the-change/issues/27

        The bug occurs when the dict keys are the same,
        so it recurses to check equality of the values,
        and one of the values is a numpy array

        '''
        o = TestModelWithVars.objects.create()
        o.vars['array'] = numpy.array([0,0])
        o.save()

        o = TestModelWithVars.objects.get()
        array = o.vars['array']

        # test that it was round-tripped without data loss
        self.assertIsInstance(array, numpy.ndarray)
        # can't use equality on numpy arrays
        self.assertEqual(list(array), [0,0])

        o.vars['array'][0] = 1
        o.save()



from __future__ import division
from otree.api import (
    BaseSubsession, BaseGroup, BasePlayer, BaseConstants, models,
    Currency
)


doc = """
Bot that doesn't submit anything. In some apps, a given player might skip all pages.
Make sure this doesn't cause a TypeError because play_round doesn't yield anything,
so it isn't considered iterable.
"""


class Constants(BaseConstants):
    name_in_url = 'bots_empty'
    players_per_group = None
    num_rounds = 1


class Subsession(BaseSubsession):
    pass


class Group(BaseGroup):
    pass


class Player(BasePlayer):
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# =============================================================================
# IMPORTS
# =============================================================================

from django.http import HttpResponseServerError

from otree.common_internal import missing_db_tables


class CheckDBMiddleware(object):

    synced = None

    def process_request(self, request):
        if not CheckDBMiddleware.synced:
            missing_tables = missing_db_tables()
            if missing_tables:
                listed_tables = missing_tables[:3]
                unlisted_tables = missing_tables[3:]
                msg = (
                    "Your database is not ready. Try running 'otree resetdb'. "
                    "(Missing tables for {}, and {} other models). "
                ).format(
                    ', '.join(listed_tables), len(unlisted_tables))
                return HttpResponseServerError(msg)
            else:
                CheckDBMiddleware.synced = True



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import absolute_import

import six

from django.db import models
from django.db.models.fields import related
from django.core import exceptions
from django.utils.translation import ugettext_lazy
from django.apps import apps


import easymoney

from idmap.models import IdMapModelBase
from .idmap import IdMapModel

import otree.common
from otree.common_internal import (
    expand_choice_tuples, get_app_label_from_import_path)
from otree.constants_internal import field_required_msg
from otree_save_the_change.mixins import SaveTheChange

# this is imported from other modules
from .serializedfields import _PickleField

class _JSONField(models.TextField):
    '''just keeping around so that Migrations don't crash'''
    pass


class OTreeModelBase(IdMapModelBase):
    def __new__(mcs, name, bases, attrs):
        meta = attrs.get("Meta")
        module = attrs.get("__module__")
        is_concrete = not getattr(meta, "abstract", False)
        app_label = getattr(meta, "app_label", "")

        if is_concrete and module and not app_label:
            if meta is None:
                meta = type("Meta", (), {})
            app_label = get_app_label_from_import_path(module)
            meta.app_label = app_label
            meta.db_table = "{}_{}".format(app_label, name.lower())
            # i think needs to be here even though it's set on base model,
            # because meta is not inherited (but not tested this)
            meta.use_strong_refs = True
            attrs["Meta"] = meta

        new_class = super().__new__(mcs, name, bases, attrs)

        # 2015-12-22: this probably doesn't work anymore,
        # since we moved _choices to views.py
        # but we can tell users they can define FOO_choices in models.py,
        # and then call it in the equivalent method in views.py
        for f in new_class._meta.fields:
            if hasattr(new_class, f.name + '_choices'):
                attr_name = 'get_%s_display' % f.name
                setattr(new_class, attr_name, make_get_display(f))

        return new_class


def get_model(*args, **kwargs):
    return apps.get_model(*args, **kwargs)


def make_get_display(field):
    def get_FIELD_display(self):
        choices = getattr(self, field.name + '_choices')()
        value = getattr(self, field.attname)
        return dict(expand_choice_tuples(choices))[value]

    return get_FIELD_display


class OTreeModel(SaveTheChange, IdMapModel, metaclass=OTreeModelBase):

    class Meta:
        abstract = True


Model = OTreeModel


class _OtreeModelFieldMixin(object):
    def fix_choices_arg(self, kwargs):
        '''allows the programmer to define choices as a list of values rather
        than (value, display_value)

        '''
        choices = kwargs.get('choices')
        if not choices:
            return
        choices = expand_choice_tuples(choices)
        kwargs['choices'] = choices

    def set_otree_properties(self, kwargs):

        # Give a `widget` argument to a model field in order to override the
        # default widget used for this field in a model form.

        # The given widget will only be used when you subclass your model form from
        # otree.forms.forms.BaseModelForm.

        self.widget = kwargs.pop('widget', None)
        self.doc = kwargs.pop('doc', None)
        self.min = kwargs.pop('min', None)
        self.max = kwargs.pop('max', None)

    def __init__(
            # list args explicitly so they show up in IDE autocomplete.
            # Hide the ones that oTree users will rarely need, like
            # db_index, primary_key, etc...
            self,
            *,
            choices=None,
            widget=None,
            initial=None,
            verbose_name=None,
            doc='',
            min=None,
            max=None,
            blank=False,
            null=True,
            help_text='',
            **kwargs):

        # ...but put them all back into kwargs so that there is a consistent
        # interface to work with them
        kwargs.update(dict(
            choices=choices,
            widget=widget,
            initial=initial,
            verbose_name=verbose_name,
            doc=doc,
            min=min,
            max=max,
            blank=blank,
            null=null,
            help_text=help_text,
        ))

        self.set_otree_properties(kwargs)
        self.fix_choices_arg(kwargs)

        # "initial" is an alias for default. in the context of oTree, 'initial'
        # is a more intuitive name. (since the user never instantiates objects
        # themselves. also, "default" could be misleading -- people could think
        # it's the default choice in the form
        if 'initial' in kwargs:
            kwargs.setdefault('default', kwargs.pop('initial'))

        # if default=None, Django will omit the blank choice from form widget
        # https://code.djangoproject.com/ticket/10792
        # that is contrary to the way oTree views blank/None values, so to
        # correct for this, we get rid of default=None args.
        # setting null=True already should make the field null
        if 'default' in kwargs and kwargs['default'] is None:
            kwargs.pop('default')

        super(_OtreeModelFieldMixin, self).__init__(**kwargs)


class _OtreeNumericFieldMixin(_OtreeModelFieldMixin):
    auto_submit_default = 0


class RealWorldCurrencyField(_OtreeNumericFieldMixin, easymoney.MoneyField):
    MONEY_CLASS = otree.common.RealWorldCurrency

    auto_submit_default = otree.common.RealWorldCurrency(0)


class CurrencyField(_OtreeNumericFieldMixin, easymoney.MoneyField):
    MONEY_CLASS = otree.common.Currency

    auto_submit_default = otree.common.Currency(0)



class BooleanField(_OtreeModelFieldMixin, models.NullBooleanField):
    # 2014/3/28: i just define the allowable choices on the model field,
    # instead of customizing the widget since then it works for any widget

    def __init__(self,
                 *,
                 choices=None,
                 widget=None,
                 initial=None,
                 verbose_name=None,
                 doc='',
                 null=True,
                 help_text='',
                 **kwargs):
        # 2015-1-19: why is this here? isn't this the default behavior?
        # 2013-1-26: ah, because we don't want the "----" (None) choice
        if choices is None:
            choices = (
                (True, ugettext_lazy('Yes')),
                (False, ugettext_lazy('No'))
            )

        # We need to store whether blank is explicitly specified or not. If
        # it's not specified explicitly (which will make it default to False)
        # we need to special case validation logic in the form field if a
        # checkbox input is used.
        self._blank_is_explicit = 'blank' in kwargs

        super(BooleanField, self).__init__(
            choices=choices,
            widget=widget,
            initial=initial,
            verbose_name=verbose_name,
            doc=doc,
            null=null,
            help_text=help_text,
            **kwargs)

        # you cant override "blank" or you will destroy the migration system
        self.allow_blank = bool(kwargs.get("blank"))

    auto_submit_default = False

    def clean(self, value, model_instance):
        if value is None and not self.allow_blank:
            raise exceptions.ValidationError(field_required_msg)
        return super(BooleanField, self).clean(value, model_instance)

    def formfield(self, *args, **kwargs):
        from otree import widgets

        is_checkbox_widget = isinstance(self.widget, widgets.CheckboxInput)
        if not self._blank_is_explicit and is_checkbox_widget:
            kwargs.setdefault('required', False)
        else:
            # this use the allow_blank for the form fields
            kwargs.setdefault('required', not self.allow_blank)

        return super(BooleanField, self).formfield(*args, **kwargs)


class AutoField(_OtreeModelFieldMixin, models.AutoField):
    pass


class BigIntegerField(
        _OtreeNumericFieldMixin, models.BigIntegerField):
    auto_submit_default = 0


class BinaryField(_OtreeModelFieldMixin, models.BinaryField):
    pass


# FIXME: CharField should never be nullable, otherwise we have to check for two
#        empty values: None and the empty string.
class CharField(_OtreeModelFieldMixin, models.CharField):
    def __init__(
            self,
            *,
            choices=None,
            widget=None,
            initial=None,
            verbose_name=None,
            doc='',
            # varchar max length doesn't affect performance or even storage
            # size; it's just for validation. so, to be easy to use,
            # there is no reason for oTree to set a short default length
            # for CharFields. The main consideration is that MySQL cannot index
            # varchar longer than 255 chars, but that is not relevant here
            # because oTree only uses indexes for fields defined in otree-core,
            # which have explicit max_lengths anyway.
            max_length=10000,
            blank=False,
            null=True,
            help_text='',
            **kwargs):
        super(CharField, self).__init__(
            choices=choices,
            widget=widget,
            initial=initial,
            verbose_name=verbose_name,
            doc=doc,
            max_length=max_length,
            blank=blank,
            null=null,
            help_text=help_text,
            **kwargs)

    auto_submit_default = ''


class CommaSeparatedIntegerField(_OtreeModelFieldMixin,
                                 models.CommaSeparatedIntegerField):
    pass


class DateField(_OtreeModelFieldMixin, models.DateField):
    pass


class DateTimeField(_OtreeModelFieldMixin, models.DateTimeField):
    pass


class DecimalField(
        _OtreeNumericFieldMixin,
        models.DecimalField):
    pass


class EmailField(_OtreeModelFieldMixin, models.EmailField):
    pass


class FileField(_OtreeModelFieldMixin, models.FileField):
    pass


class FilePathField(_OtreeModelFieldMixin, models.FilePathField):
    pass


class FloatField(
        _OtreeNumericFieldMixin,
        models.FloatField):
    pass


class IntegerField(
        _OtreeNumericFieldMixin, models.IntegerField):
    pass


class GenericIPAddressField(_OtreeModelFieldMixin,
                            models.GenericIPAddressField):
    pass


class PositiveIntegerField(
        _OtreeNumericFieldMixin,
        models.PositiveIntegerField):
    pass


class PositiveSmallIntegerField(
        _OtreeNumericFieldMixin,
        models.PositiveSmallIntegerField):
    pass


class SlugField(_OtreeModelFieldMixin, models.SlugField):
    pass


class SmallIntegerField(
        _OtreeNumericFieldMixin, models.SmallIntegerField):
    pass


class TextField(_OtreeModelFieldMixin, models.TextField):
    auto_submit_default = ''


class TimeField(_OtreeModelFieldMixin, models.TimeField):
    pass


class URLField(_OtreeModelFieldMixin, models.URLField):
    pass


ForeignKey = models.ForeignKey
ManyToOneRel = related.ManyToOneRel
ManyToManyField = models.ManyToManyField
OneToOneField = models.OneToOneField



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import glob
import inspect
import io
import os
import types
from functools import wraps
from importlib import import_module

from django.apps import apps
from django.conf import settings
from django.core.checks import register, Error, Warning
from django.template import Template
from django.template import TemplateSyntaxError

from otree.api import (
    BasePlayer, BaseGroup, BaseSubsession, Currency, WaitPage, Page)
from otree.common_internal import _get_all_configs


class Rules(object):
    """A helper class incapsulating common checks.

    Usage:
        rules = Rules(app_config, errors_list)

        # various rule checks, see below for list of rules
        rules.file_exists('some_file.py')
        ...

        # custom checks
        if <your-condition>:
            rules.push_error('...', id='...')

        # using checks as guards
        if rules.module_exists('tests'):
            tests = rules.get_module('tests') # won't fail
            ...

    """

    common_buffer = {}

    def __init__(self, config, errors, id=None):
        self.config = config
        self.errors = errors
        self.id = id

    def rule(meth):
        '''
        wrapper to return True if the method doesn't return anything.
        and False if it does return something.
        '''
        @wraps(meth)
        def wrapper(self, *args, **kwargs):
            res = meth(self, *args, **kwargs)
            if res:
                self.errors.append(res)
                return False
            else:
                return True
        return wrapper

    def error(self, title, **kwargs):
        kwargs.setdefault('obj', self.config.label)
        kwargs.setdefault('id', self.id)
        return Error(title, **kwargs)

    def push_error(self, title, **kwargs):
        return self.errors.append(self.error(title, **kwargs))

    def warning(self, title, **kwargs):
        kwargs.setdefault('obj', self.config.label)
        kwargs.setdefault('id', self.id)
        return Warning(title, **kwargs)

    def push_warning(self, title, **kwargs):
        return self.errors.append(self.warning(title, **kwargs))

    # Helper meythods

    def get_path(self, name):
        return os.path.join(self.config.path, name)

    def get_rel_path(self, name):
        basepath = os.getcwd()
        return os.path.relpath(name, basepath)

    def get_module(self, name):
        return import_module(self.config.name + '.' + name)

    def get_module_attr(self, module, name):
        if not isinstance(module, types.ModuleType):
            module = self.get_module(module)
        return getattr(module, name)

    def get_template_names(self):
        path = self.get_path('templates')
        template_names = []
        for root, dirs, files in os.walk(path):
            for filename in [f for f in files if f.endswith('.html')]:
                template_names.append(os.path.join(root, filename))
        return template_names

    # Rule methods

    @rule
    def file_exists(self, filename):
        if not os.path.isfile(self.get_path(filename)):
            return self.error('No "%s" file found in game folder' % filename)

    @rule
    def dir_exists(self, filename):
        if not os.path.isdir(self.get_path(filename)):
            msg = 'No "%s" directory found in game folder' % filename
            return self.error(msg)

    @rule
    def model_exists(self, name):
        try:
            self.config.get_model(name)
        except LookupError:
            return self.error('Model "%s" not defined' % name)

    @rule
    def module_exists(self, module):
        try:
            module = self.get_module(module)
        except ImportError as e:
            return self.error('Can\'t import module "%s": %s' % (module, e))

    @rule
    def class_exists(self, module, name):
        module = self.get_module(module)
        cls = getattr(module, name, None)
        if not inspect.isclass(cls):
            msg = 'No class "%s" in module "%s"' % (name, module.__name__)
            return self.error(msg)

    @rule
    def template_has_valid_syntax(self, template_name):
        from otree.checks.templates import has_valid_encoding
        from otree.checks.templates import format_source_snippet

        # Only test files that are valid templates.
        if not has_valid_encoding(template_name):
            return

        try:
            with io.open(template_name, 'r', encoding='utf8') as f:
                Template(f.read())
        except (IOError, OSError):
            pass
        except TemplateSyntaxError as error:
            # The django_template_source attribute will only be available on
            # DEBUG = True.
            if hasattr(error, 'django_template_source'):
                template_source, position = error.django_template_source
                snippet = format_source_snippet(
                    template_source.source,
                    arrow_position=position[0])
                message = (
                    'Template syntax error in {template}\n'
                    '\n'
                    '{snippet}\n'
                    '\n'
                    'Error: {error}'.format(
                        template=template_name,
                        error=error,
                        snippet=snippet))
            else:
                message = (
                    'Template syntax error in {template}\n'
                    'Error: {error}\n'
                    'Set "DEBUG = True" to see more details.'.format(
                        template=template_name, error=error))
            return self.error(message)

    @rule
    def template_has_no_dead_code(self, template_name):
        from otree.checks.templates import get_unreachable_content
        from otree.checks.templates import has_valid_encoding

        # Only test files that are valid templates.
        if not has_valid_encoding(template_name):
            return

        try:
            with io.open(template_name, 'r', encoding='utf8') as f:
                compiled_template = Template(f.read())
        except (IOError, OSError, TemplateSyntaxError):
            # Ignore errors that occured during file-read or compilation.
            return

        def format_content(text):
            text = text.strip()
            lines = text.splitlines()
            lines = ['> {0}'.format(line) for line in lines]
            return '\n'.join(lines)

        contents = get_unreachable_content(compiled_template)
        content_bits = '\n\n'.join(
            format_content(bit)
            for bit in contents)
        if contents:
            return self.error(
                'Template contains the following text outside of a '
                '{% block %}. This text will never be displayed.'
                '\n\n' + content_bits,
                obj=os.path.join(self.config.label,
                                 self.get_rel_path(template_name)))

    @rule
    def template_has_valid_encoding(self, template_name):
        from otree.checks.templates import has_valid_encoding

        if not has_valid_encoding(template_name):
            return self.error(
                'The template {template} is not UTF-8 encoded. '
                'Please configure your text editor to always save files '
                'as UTF-8. Then open the file and save it again.'
                .format(template=self.get_rel_path(template_name)))


def register_rules(tags=(), id=None, once_per_project=False):
    """Transform a function based on rules, to a something
    django.core.checks.register takes.
    Passes Rules instance as first argument.
    """
    def decorator(func):
        @register(*tags)
        @wraps(func)
        def wrapper(app_configs, **kwargs):
            if once_per_project:
                # some checks should only be run once, not for each app
                app_configs = [apps.get_app_config('otree')]
            else:
                # if app_configs list is given (e.g. otree check app1 app2), run on those
                # if it's None, run on all apps
                # (system check API requires this)
                app_configs = app_configs or _get_all_configs()
            errors = []
            for config in app_configs:
                rules = Rules(config, errors, id=id)
                func(rules, **kwargs)
            return errors
        return wrapper
    return decorator


# Checks
@register_rules(id='otree.E001')
def files(rules, **kwargs):
    rules.file_exists('models.py')
    rules.file_exists('views.py')

    if os.path.isdir(rules.get_path('templates')):
        # check for files in templates, but not in templates/<label>
        misplaced_templates = set(glob.glob(
            os.path.join(rules.get_path('templates'), '*.html')
        ))
        misplaced_templates.discard(rules.config.label)
        if misplaced_templates:
            hint = (
                'Move template files from "{app}/templates/" '
                'to "{app}/templates/{app}" subfolder'.format(
                    app=rules.config.label)
            )
            rules.push_error(
                "Templates files in app's root template directory",
                hint=hint, id='otree.E001'
            )


base_model_attrs = {
    'Player': set(dir(BasePlayer)),
    'Group': set(dir(BaseGroup)),
    'Subsession': set(dir(BaseSubsession)),
}

model_field_substitutes = {
    int: 'IntegerField',
    float: 'FloatField',
    bool: 'BooleanField',
    str: 'CharField',
    Currency: 'CurrencyField'
}


@register_rules(id='otree.E002')
def model_classes(rules, **kwargs):
    rules.model_exists('Subsession')
    rules.model_exists('Group')
    rules.model_exists('Player')

    config = rules.config
    Player = config.get_model('Player')
    Group = config.get_model('Group')
    Subsession = config.get_model('Subsession')

    for Model in [Player, Group, Subsession]:
        for attr in dir(Model):
            if attr not in base_model_attrs[Model.__name__]:
                try:
                    _type = type(getattr(Model, attr))
                except AttributeError:
                    # I got "The 'q_country' attribute can only be accessed
                    # from Player instances."
                    # can just filter/ignore these.
                    pass
                else:
                    if _type in model_field_substitutes.keys():
                        rules.push_warning(
                            '{} has attribute "{}", which is not a model field, '
                            'and will therefore not be saved '
                            'to the database.'.format(
                                Model.__name__, attr
                            ),
                            hint='Consider changing to "{} = models.{}(...)"'.format(attr, model_field_substitutes[_type])
                        )
                    # if people just need an iterable of choices for a model field,
                    # they should use a tuple, not list or dict
                    if _type in {list, dict, set}:
                        warning = (
                            '{ModelName}.{attr} is a {type_name}. '
                            'Modifying it during a session (e.g. appending or setting values) '
                            'will have unpredictable results; '
                            'you should use '
                            'session.vars or participant.vars instead. '
                            'Or, if this {type_name} is read-only, '
                            "then it's recommended to move it outside of this class "
                            '(e.g. put it in Constants).'
                        ).format(ModelName=Model.__name__,
                                 attr=attr,
                                 type_name=_type.__name__)

                        rules.push_warning(warning)


@register_rules(id='otree.E003')
def constants(rules, **kwargs):
    cond = (
        rules.module_exists('models') and
        rules.class_exists('models', 'Constants')
    )
    if cond:
        Constants = rules.get_module_attr('models', 'Constants')
        attrs = ['name_in_url', 'players_per_group', 'num_rounds']
        for attr_name in attrs:
            if not hasattr(Constants, attr_name):
                msg = "models.py: 'Constants' class needs to define '{}'"
                rules.push_error(msg.format(attr_name))
        ppg = Constants.players_per_group
        if ppg == 0 or ppg == 1:
            rules.push_error(
                "models.py: 'Constants.players_per_group' cannot be {}. You "
                "should set it to None, which makes the group "
                "all players in the subsession.".format(ppg)
            )


@register_rules(id='otree.E004')
def pages_function(rules, **kwargs):
    if rules.module_exists('views'):
        views_module = rules.get_module('views')
        try:
            page_list = views_module.page_sequence
        except:
            rules.push_error('views.py is missing the variable page_sequence.')
            return
        else:
            for i, ViewCls in enumerate(page_list):
                # there is no good reason to include Page in page_sequence.
                # As for WaitPage: even though it works fine currently
                # and can save the effort of subclassing,
                # we should restrict it, because:
                # - one user had "class WaitPage(Page):".
                # - if someone makes "class WaitPage(WaitPage):", they might
                #   not realize why it's inheriting the extra behavior.
                # overall, I think the small inconvenience of having to subclass
                # once per app
                # is outweighed by the unexpected behavior if someone subclasses
                # it without understanding inheritance.
                # BUT: built-in Trust game has a wait page called WaitPage.
                # need to get rid of that first.
                if ViewCls.__name__ == 'Page':
                    msg = (
                        "views.py: page_sequence cannot contain "
                        "a class called 'Page'. You should subclass Page "
                        "and give your page a different name."
                    )
                    rules.push_error(msg)
                if issubclass(ViewCls, WaitPage):
                    if hasattr(ViewCls, 'before_next_page'):
                        rules.push_error(
                            'views.py: "{}" defines before_next_page, '
                            'which is not valid on wait pages.'.format(
                                ViewCls.__name__)
                        )
                    if ViewCls.group_by_arrival_time:
                        if i > 0:
                            rules.push_error(
                                'views.py: "{}" has group_by_arrival_time=True, so '
                                'it must be placed first in page_sequence.'.format(
                                    ViewCls.__name__))
                        if ViewCls.wait_for_all_groups:
                            rules.push_error(
                                'views.py: "{}" has group_by_arrival_time=True, so '
                                'it cannot have wait_for_all_groups=True also.'.format(
                                    ViewCls.__name__))
                    # alternative technique is to not define the method on WaitPage
                    # and then use hasattr, but I want to keep all complexity
                    # out of views.abstract
                    elif (ViewCls.get_players_for_group != WaitPage.get_players_for_group):
                        rules.push_error(
                            'views.py: "{}" defines get_players_for_group, '
                            'but in order to use this method, you must set '
                            'group_by_arrival_time=True'.format(
                                ViewCls.__name__))
                elif issubclass(ViewCls, Page):
                    pass # ok
                else:
                    msg = 'views.py: "{}" is not a valid page'.format(ViewCls)
                    rules.push_error(msg)


@register_rules(id='otree.E005')
def templates_have_no_dead_code(rules, **kwargs):
    for template_name in rules.get_template_names():
        rules.template_has_no_dead_code(template_name)


@register_rules(id='otree.E006', once_per_project=True)
def unique_sessions_names(rules, **kwargs):
    already_seen = set()
    for st in settings.SESSION_CONFIGS:
        st_name = st["name"]
        if st_name in already_seen:
            msg = "Duplicate SESSION_CONFIG name '{}'".format(st_name)
            rules.push_error(msg)
        else:
            already_seen.add(st_name)


@register_rules(id='otree.E009', once_per_project=True)
def unique_room_names(rules, **kwargs):
    already_seen = set()
    for room in getattr(settings, 'ROOMS', []):
        room_name = room["name"]
        if room_name in already_seen:
            msg = "Duplicate ROOM name '{}'".format(room_name)
            rules.push_error(msg)
        else:
            already_seen.add(room_name)


@register_rules(id='otree.E007')
def template_encoding(rules, **kwargs):
    for template_name in rules.get_template_names():
        rules.template_has_valid_encoding(template_name)


@register_rules(id='otree.E008')
def templates_have_valid_syntax(rules, **kwargs):
    for template_name in rules.get_template_names():
        rules.template_has_valid_syntax(template_name)


# TODO: startapp should pass validation checks



# -*- coding: utf-8 -*-
from __future__ import division, absolute_import
from . import models
from otree.api import WaitPage
from tests.utils import BlankTemplatePage as Page
from .models import Constants


class MyWait(WaitPage):
    def after_all_players_arrive(self):
        for p in self.group.get_players():
            p.set_to_false = False


class ShouldBeSkipped(Page):
    def is_displayed(self):
        return self.player.set_to_false

    def vars_for_template(self):
        raise AssertionError('This page should not be shown')


class Page2(Page):
    pass


page_sequence = [
    MyWait,
    ShouldBeSkipped,
    Page2
]



from __future__ import division
from otree.api import (
    BaseSubsession, BaseGroup, BasePlayer, BaseConstants, models
)
import itertools

doc = "Testing get_players_for_group()"

class Constants(BaseConstants):
    name_in_url = 'group_by_arrival_time_custom'
    players_per_group = None
    num_rounds = 2


class Subsession(BaseSubsession):
    def before_session_starts(self):
        types = itertools.cycle(['A','B'])
        for p in self.get_players():
            p.type = next(types)


class Group(BaseGroup):
    pass


class Player(BasePlayer):
    type = models.CharField()



import contextlib
import os
import shutil
import sys
import tempfile
import warnings

import django.test
import splinter.browser

from six import StringIO
from six.moves import urllib

from django.conf import settings
from django.core.management import call_command
from django.test.utils import override_settings

from otree.api import Page
from otree.session import create_session
from otree.bots.runner import session_bot_runner_factory


class BlankTemplatePage(Page):
    template_name = 'global/BlankTemplatePage.html'


@contextlib.contextmanager
def add_path(path):
    """
    ::

        with add_path(new_sys_path):
            import strange_module
    """
    sys.path.insert(0, path)
    yield
    sys.path.pop(sys.path.index(path))


@contextlib.contextmanager
def capture_stdout(target=None):
    original = sys.stdout
    if target is None:
        target = StringIO()
    sys.stdout = target
    yield target
    target.seek(0)
    sys.stdout = original


@contextlib.contextmanager
def capture_stderr(target=None):
    original = sys.stderr
    if target is None:
        target = StringIO()
    sys.stderr = target
    yield target
    target.seek(0)
    sys.stderr = original


@contextlib.contextmanager
def cd(directory):
    """
    ::

        with cd(new_cwd):
            os.walk('.')
    """
    old_path = os.getcwd()
    os.chdir(directory)
    yield
    os.chdir(old_path)


@contextlib.contextmanager
def dummyapp(app):
    """
    Creates a new game app inside a temporary directory, adds this to the path
    and puts the app into the INSTALLED_APPS and INSTALLED_OTREE_APPS settings.

    It cleans the filesystem up afterwards.
    """
    tmpdir = tempfile.mkdtemp()
    app_path = os.path.join(tmpdir, app)

    os.mkdir(app_path)
    with capture_stdout():
        call_command('startapp', app, app_path)

    new_apps = list(settings.INSTALLED_APPS) + [app]
    new_otree_apps = list(settings.INSTALLED_OTREE_APPS) + [app]
    with add_path(tmpdir):
        with override_settings(
                INSTALLED_APPS=new_apps,
                INSTALLED_OTREE_APPS=new_otree_apps):
            yield app_path

        shutil.rmtree(tmpdir)
        if os.path.exists(tmpdir):
            shutil.rmtree(tmpdir)


def get_path(test_client_response, if_no_redirect):
    try:
        url = test_client_response.redirect_chain[-1][0]
    except IndexError:
        return if_no_redirect
    else:
        return urllib.parse.urlsplit(url).path


class OTreeTestClient(django.test.client.Client):

    def login(self):
        return super(OTreeTestClient, self).login(
            username=settings.ADMIN_USERNAME, password=settings.ADMIN_PASSWORD)


class TestCase(django.test.TestCase):

    client_class = OTreeTestClient

    def assertWarns(self, warning, callable, *args, **kwds):
        with warnings.catch_warnings(record=True) as warning_list:
            warnings.simplefilter('always')

            callable(*args, **kwds)

        condition = any(item.category == warning for item in warning_list)
        msg = "'{}' not warned".format(str(warning))
        self.assertTrue(condition, msg)


class OTreePhantomBrowser(splinter.browser.PhantomJSWebDriver):

    # splinter.Browser is actually a function, not a class
    def __init__(self, live_server_url):
        self.live_server_url = live_server_url
        super().__init__()

            #user_agent='Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/59.0.3071.115 Safari/537.36'
        #)

    def go(self, relative_url):
        self.visit(self.live_server_url + relative_url)


def run_bots(session_config_name, num_participants):
    session = create_session(
        session_config_name, num_participants=num_participants, use_cli_bots=True)
    bot_runner = session_bot_runner_factory(session)
    bot_runner.play()



import otree.forms
from otree import widgets
from otree.db import models

from .utils import TestCase
from .models import CurrencyFieldTestModel, SimpleModel, FormFieldModel


class TestModelForm(otree.forms.ModelForm):
    class Meta:
        model = FormFieldModel
        exclude = ()


class WidgetArgumentTests(TestCase):

    def test_widget_argument(self):
        self.assertIsInstance(
            TestModelForm.base_fields['char'].widget, otree.forms.TextInput
        )
        self.assertIsInstance(
            TestModelForm.base_fields['alt_date_time'].widget,
            otree.forms.SplitDateTimeWidget
        )
        self.assertIsInstance(
            TestModelForm.base_fields['text'].widget, otree.forms.Textarea
        )
        self.assertIsInstance(
            TestModelForm.base_fields['alt_text'].widget,
            otree.forms.TextInput
        )


class ModelTests(TestCase):

    def test_get_FIELD_display(self):
        obj = SimpleModel(name='bob')
        self.assertEqual(obj.get_name_display(), 'BOB')


class BooleanFieldTests(TestCase):

    def test_normal_widget_is_required_depending_on_blank(self):

        modelfield = models.BooleanField()
        formfield = modelfield.formfield()

        self.assertTrue(formfield.required)

        modelfield = models.BooleanField(blank=True)
        formfield = modelfield.formfield()

        self.assertFalse(formfield.required)

    def test_checkbox_form_field_is_not_required(self):

        modelfield = models.BooleanField(widget=widgets.CheckboxInput())
        formfield = modelfield.formfield()

        self.assertFalse(formfield.required)

    def test_checkbox_form_field_is_required_if_blank_is_set(self):

        # blank=True means en empty checkbox is ok.

        modelfield = models.BooleanField(
            widget=widgets.CheckboxInput(),
            blank=True)
        formfield = modelfield.formfield()

        self.assertFalse(formfield.required)

        # blank=False means the checkbox must be checked (is required).

        modelfield = models.BooleanField(
            widget=widgets.CheckboxInput(),
            blank=False)
        formfield = modelfield.formfield()

        self.assertTrue(formfield.required)


class CurrencyFieldTests(TestCase):

    def test_default_value_zero_in_modelform(self):
        # Make sure that a value of 0.00 is included in rendered output. It is
        # a falsy value if it's a Decimal instance. So this might fail if the
        # value is not converted to string prior to rendering in the form. See
        # https://github.com/oTree-org/otree-core/issues/326

        class CurrencyFieldTestModelForm(otree.forms.ModelForm):
            class Meta:
                model = CurrencyFieldTestModel
                exclude = ()

        form = CurrencyFieldTestModelForm()
        rendered = str(form['currency_with_default_value_zero'])
        self.assertTrue('value="0.00"' in rendered)






from django.core.management import call_command
from django.core.urlresolvers import reverse
from otree.models import Session
from .utils import TestCase
import splinter

class TestAdminReport(TestCase):

    def setUp(self):
        call_command('create_session', 'admin_report', "1")
        self.session = Session.objects.get()
        self.browser = splinter.Browser('django') # type: splinter.Browser


    def test_load(self):
        url = reverse('AdminReport', args=[self.session.code])
        browser = self.browser
        browser.visit(url)

        for substring in ['42', '43', 'round 2']:
            self.assertTrue(browser.is_text_present(substring))

        browser.fill('app_name', 'tests.admin_report')
        browser.fill('round_number', '1')
        button = browser.find_by_id('submit')
        # Interact with elements
        button.click()
        self.assertIn('round 1', browser.html)



import os
from django.core.management.commands import startproject

import six

import otree
from otree.management.cli import pypi_updates_cli


class Command(startproject.Command):
    help = ("Creates a new oTree project.")

    def add_arguments(self, parser):
        super(Command, self).add_arguments(parser)
        ahelp = (
            'Tells the command to NOT prompt the user for '
            'input of any kind.')
        parser.add_argument(
            '--noinput', action='store_false', dest='interactive',
            default=True, help=ahelp)

    def handle(self, *args, **options):
        if options["interactive"]:
            answer = None
            while not answer or answer not in "yn":
                answer = six.moves.input("Include sample games? (y or n): ")
                if not answer:
                    answer = "y"
                    break
                else:
                    answer = answer[0].lower()
        else:
            answer = 'n'
        self.core_project_template_path = os.path.join(
                os.path.dirname(otree.__file__), 'project_template')
        if answer == "y":
            project_template_path = (
                "https://github.com/oTree-org/oTree/archive/master.zip")
        else:
            project_template_path = self.core_project_template_path
        if options.get('template', None) is None:
            options['template'] = project_template_path
        super(Command, self).handle(*args, **options)

        try:
            pypi_updates_cli()
        except:
            pass
        print('Created project folder.')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import logging
import channels
import json
import django.test
from django.core.urlresolvers import reverse

from otree import constants_internal
import otree.common_internal
from otree.common_internal import (
    random_chars_8, random_chars_10, get_admin_secret_code,
    get_app_label_from_name
)
from otree.db import models
from otree.models_concrete import ParticipantToPlayerLookup, RoomToSession
from django.template.loader import get_template
from django.template import TemplateDoesNotExist
from .varsmixin import ModelWithVars
logger = logging.getLogger('otree')

client = django.test.Client()

ADMIN_SECRET_CODE = get_admin_secret_code()

class Session(ModelWithVars):

    class Meta:
        app_label = "otree"
        # if i don't set this, it could be in an unpredictable order
        ordering = ['pk']

    _pickle_fields = ['vars', 'config']
    config = models._PickleField(default=dict, null=True)  # type: dict

    # label of this session instance
    label = models.CharField(
        max_length=300, null=True, blank=True,
        help_text='For internal record-keeping')

    experimenter_name = models.CharField(
        max_length=300, null=True, blank=True,
        help_text='For internal record-keeping')

    ready = models.BooleanField(default=False)

    code = models.CharField(
        default=random_chars_8,
        max_length=16,
        # set non-nullable, until we make our CharField non-nullable
        null=False,
        unique=True,
        doc="Randomly generated unique identifier for the session.")

    mturk_HITId = models.CharField(
        max_length=300, null=True, blank=True,
        help_text='Hit id for this session on MTurk')
    mturk_HITGroupId = models.CharField(
        max_length=300, null=True, blank=True,
        help_text='Hit id for this session on MTurk')

    # since workers can drop out number of participants on server should be
    # greater than number of participants on mturk
    # value -1 indicates that this session it not intended to run on mturk
    mturk_num_participants = models.IntegerField(
        default=-1,
        help_text="Number of participants on MTurk")

    mturk_use_sandbox = models.BooleanField(
        default=True,
        help_text="Should this session be created in mturk sandbox?")

    archived = models.BooleanField(
        default=False,
        db_index=True,
        doc=("If set to True the session won't be visible on the "
             "main ViewList for sessions"))

    comment = models.TextField(blank=True)

    _anonymous_code = models.CharField(
        default=random_chars_10, max_length=10, null=False, db_index=True)

    _pre_create_id = models.CharField(max_length=255, db_index=True, null=True)

    use_browser_bots = models.BooleanField(default=False)

    # if the user clicks 'start bots' twice, this will prevent the bots
    # from being run twice.
    _cannot_restart_bots = models.BooleanField(default=False)
    _bots_finished = models.BooleanField(default=False)
    _bots_errored = models.BooleanField(default=False)
    _bot_case_number = models.PositiveIntegerField()

    is_demo = models.BooleanField(default=False)

    # whether SOME players are bots
    has_bots = models.BooleanField(default=False)

    _admin_report_app_names = models.TextField(default='')
    _admin_report_num_rounds = models.CharField(default='', max_length=255)

    num_participants = models.PositiveIntegerField()

    def __unicode__(self):
        return self.code

    @property
    def participation_fee(self):
        '''This method is deprecated from public API,
        but still useful internally (like data export)'''
        return self.config['participation_fee']

    @property
    def real_world_currency_per_point(self):
        '''This method is deprecated from public API,
        but still useful internally (like data export)'''
        return self.config['real_world_currency_per_point']

    def is_for_mturk(self):
        return (not self.is_demo) and (self.mturk_num_participants > 0)

    def get_subsessions(self):
        lst = []
        app_sequence = self.config['app_sequence']
        for app in app_sequence:
            models_module = otree.common_internal.get_models_module(app)
            subsessions = models_module.Subsession.objects.filter(
                session=self
            ).order_by('round_number')
            lst.extend(list(subsessions))
        return lst

    def get_participants(self):
        return self.participant_set.all()

    def _create_groups_and_initialize(self):
        # group_by_arrival_time_time code used to be here
        for subsession in self.get_subsessions():
            subsession._create_groups()
            subsession.before_session_starts()
            subsession.creating_session()
            subsession.save()

    def mturk_requester_url(self):
        if self.mturk_use_sandbox:
            requester_url = (
                "https://requestersandbox.mturk.com/mturk/manageHITs"
            )
        else:
            requester_url = "https://requester.mturk.com/mturk/manageHITs"
        return requester_url

    def mturk_worker_url(self):
        if self.mturk_use_sandbox:
            return (
                "https://workersandbox.mturk.com/mturk/preview?groupId={}"
            ).format(self.mturk_HITGroupId)
        return (
            "https://www.mturk.com/mturk/preview?groupId={}"
        ).format(self.mturk_HITGroupId)

    def advance_last_place_participants(self):

        participants = self.get_participants()

        # in case some participants haven't started
        unvisited_participants = []
        for p in participants:
            if p._index_in_pages == 0:
                unvisited_participants.append(p)
                client.get(p._start_url(), follow=True)

        if unvisited_participants:
            # that's it -- just visit the start URL, advancing by 1
            return

        last_place_page_index = min([p._index_in_pages for p in participants])
        last_place_participants = [
            p for p in participants
            if p._index_in_pages == last_place_page_index
        ]

        for p in last_place_participants:
            try:
                current_form_page_url = p._current_form_page_url
                if current_form_page_url:
                    resp = client.post(
                        current_form_page_url,
                        data={
                            constants_internal.timeout_happened: True,
                            constants_internal.admin_secret_code: ADMIN_SECRET_CODE
                        },
                        follow=True
                    )
                    # not sure why, but many users are getting HttpResponseNotFound
                    if resp.status_code >= 400:
                        msg = ('Submitting page {} failed, '
                            'returned HTTP status code {}.'.format(
                                current_form_page_url, resp.status_code
                            ))
                        content = resp.content
                        if len(content) < 600:
                            msg += ' response content: {}'.format(content)
                        raise AssertionError(msg)

                else:
                    # it's possible that the slowest user is on a wait page,
                    # especially if their browser is closed.
                    # because they were waiting for another user who then
                    # advanced past the wait page, but they were never
                    # advanced themselves.
                    start_url = p._start_url()
                    resp = client.get(start_url, follow=True)
            except:
                logging.exception("Failed to advance participants.")
                raise


            # do the auto-advancing here,
            # rather than in increment_index_in_pages,
            # because it's only needed here.
            channels.Group(
                'auto-advance-{}'.format(p.code)
            ).send(
                {'text': json.dumps(
                    {'auto_advanced': True})}
            )

    def pages_auto_reload_when_advanced(self):
        # keep it enable until I determine
        # (a) the usefulness of the feature
        # (b) the impact on performance
        return True
        # return settings.DEBUG or self.is_demo

    def build_participant_to_player_lookups(self):
        subsession_app_names = self.config['app_sequence']

        views_modules = {}
        for app_name in subsession_app_names:
            views_modules[app_name] = (
                otree.common_internal.get_views_module(app_name))

        def views_module_for_player(player):
            return views_modules[player._meta.app_config.name]

        records_to_create = []

        for participant in self.get_participants():
            page_index = 0
            for player in participant.get_players():
                for View in views_module_for_player(player).page_sequence:
                    page_index += 1
                    records_to_create.append(
                        ParticipantToPlayerLookup(
                            participant=participant,
                            page_index=page_index,
                            app_name=player._meta.app_config.name,
                            player_pk=player.pk,
                            subsession_pk=player.subsession.pk,
                            session_pk=self.pk,
                            url=reverse(View.url_name(),
                                        args=[participant.code, page_index]))
                    )

            # technically could be stored at the session level
            participant._max_page_index = page_index
            participant.save()
        ParticipantToPlayerLookup.objects.bulk_create(records_to_create)

    def get_room(self):
        from otree.room import ROOM_DICT
        try:
            room_name = RoomToSession.objects.get(session=self).room_name
            return ROOM_DICT[room_name]
        except RoomToSession.DoesNotExist:
            return None

    def _get_payoff_plus_participation_fee(self, payoff):
        '''For a participant who has the given payoff,
        return their payoff_plus_participation_fee
        Useful to define it here, for data export
        '''

        return (
            self.config['participation_fee'] +
            payoff.to_real_world_currency(self)
        )

    def _set_admin_report_app_names(self):

        admin_report_app_names = []
        num_rounds_list = []
        for app_name in self.config['app_sequence']:
            models_module = otree.common_internal.get_models_module(app_name)
            app_label = get_app_label_from_name(app_name)
            try:
                get_template('{}/AdminReport.html'.format(app_label))
                admin_report_app_names.append(app_name)
                num_rounds_list.append(models_module.Constants.num_rounds)
            except TemplateDoesNotExist:
                pass
        self._admin_report_app_names = ';'.join(admin_report_app_names)
        self._admin_report_num_rounds = ';'.join(str(n) for n in num_rounds_list)

    def _admin_report_apps(self):
        return self._admin_report_app_names.split(';')

    def _admin_report_num_rounds_list(self):
        return [int(num) for num in self._admin_report_num_rounds.split(';')]

    def has_admin_report(self):
        return bool(self._admin_report_app_names)



from .utils import TestCase
import otree.session
from django.core.urlresolvers import reverse
import django.test
from otree.api import Currency
import otree.db.idmap

class TestPayoff(TestCase):

    def test_payoff(self):
        with self.settings(USE_POINTS=True):
            # Currency.DECIMAL_PLACES needs to be patched because the setting
            # is determined at startup, and does not change if you patch
            # USE_POINTS

            DECIMAL_PLACES_ORIGINAL_VALUE = Currency.DECIMAL_PLACES
            Currency.DECIMAL_PLACES = 2
            try:
                self.helper()
            finally:
                Currency.DECIMAL_PLACES = DECIMAL_PLACES_ORIGINAL_VALUE

    def helper(self):

        session = otree.session.create_session(
            session_config_name='two_rounds_1p',
            num_participants=1,
        )

        # need to activate cache after creating a session
        # because inside create_session, cache is deactivated
        with otree.db.idmap.use_cache():

            # for some reason id=1 test fails, because the session only has
            # participant with id=2. ah, that makes sense. even if the DB
            # is truncated, PKs will still be incremented, i think.
            participant = session.participant_set.first()

            round_players = participant.get_players()

            round_payoff = Currency(13)

            round_players[0].payoff = round_payoff
            round_players[1].payoff = round_payoff

        payoff = participant.payoff
        self.assertEqual(payoff, 2*round_payoff)

        participation_fee = session.config['participation_fee']
        self.assertEqual(participation_fee, 1.25)

        real_world_currency_per_point = session.config['real_world_currency_per_point']
        self.assertEqual(real_world_currency_per_point, 0.5)

        payoff_in_real_world_currency = payoff * real_world_currency_per_point

        payoff_plus_participation_fee = participant.payoff_plus_participation_fee()

        self.assertEqual(
            payoff_plus_participation_fee,
            payoff_in_real_world_currency + participation_fee)

        payments_url = reverse('SessionPayments', args=[session.code])

        client = django.test.Client()
        resp = client.get(payments_url)
        html = resp.content.decode('utf-8')
        for amount in [
            '\u20ac1.25', # participation fee
            '\u20ac13.00', # participant.payoff
            '\u20ac14.25' # base plus participant.payoff
        ]:
            self.assertIn(amount, html)



from otree.api import (
    BaseSubsession, BaseGroup, BasePlayer, BaseConstants, models,
    Currency as c
)

doc = '''
THIS GAME DOES NOT ACTUALLY RUN, and should not be included in session config.
It's just to testing that autocomplete works properly when editing these source
files in PyCharm.
'''

class Constants(BaseConstants):
    name_in_url = 'autocomplete'
    players_per_group = None
    num_rounds = 2


class Subsession(BaseSubsession):

    # should auto-complete if I start typing the method name
    def vars_for_admin_report(self):
        pass

    def before_session_starts(self):
        for g in self.get_group_matrix():
            for p in g:
                # autocomplete
                _ = p.id_in_group

                # no yellow
                _ = p.f_bool

class Group(BaseGroup):
    f_posint = models.PositiveIntegerField()

    def foo(self):
        # method should autocomplete
        x = self.get_player_by_id(1)

        # .id_in_group should autocomplete
        _ = x.id_in_group

        # no yellow highlight
        _ = x.f_currency
        x.f()

        for p in self.get_players():
            _ = p.id_in_group


class Player(BasePlayer):
    f_currency = models.CurrencyField(choices=[1,2])
    f_int = models.IntegerField(
        blank=False, # should be auto-completed
        db_index=True # should NOT be auto-completed
    )
    f_bool = models.BooleanField()
    f_char = models.CharField()

    def no_yellow_errors(self):
        '''None of these should be highlighted in yellow'''
        _ = self.f_currency + 1
        _ = self.f_currency + c(1)
        _ = c(1) + 1
        _ = c(1) / c(1)
        _ = self.f_int + 1
        _ = self.f_int + c(1)
        _ = self.f_bool + 1
        _ = self.group.f_posint

    def foo(self):
        # each of these should autocomplete
        self.participant.vars.clear()
        _ = self.session.config

        for p in self.in_all_rounds():
            # autocomplete
            _ = p.id_in_group

            # no yellow
            _ = p.f_bool


    def f(self): pass


from django.core.management import call_command
from otree.models.session import Session
from otree.bots.bot import ParticipantBot
from .utils import TestCase


# use a wrapper so that unittest doesn't find this base class at the
# module level, otherwise it will run it.
class WrapperToHideClass:
    class BaseTestCase(TestCase):

        def advance_to_end(self):

            session = self.session
            participants = session.get_participants()
            max_page_index = participants[0]._max_page_index

            # once to open the start links, then num_pages to get to last page
            for x in range(max_page_index + 1):
                session.advance_last_place_participants()

            participants = session.get_participants()
            for p in participants:
                self.assertGreaterEqual(p._index_in_pages, max_page_index)

        def test_advance_to_end(self):
            self.advance_to_end()

        def test_some_already_started(self):
            participants = self.session.get_participants()
            p1 = participants[0]

            bot = ParticipantBot(p1, load_player_bots=False)
            bot.open_start_url()
            self.advance_to_end()


class TestAdvanceSlowest(WrapperToHideClass.BaseTestCase):
    def setUp(self):
        call_command('create_session', 'advance_slowest', "2")
        self.session = Session.objects.get()


class TestAdvanceSlowestWaitPageFirst(WrapperToHideClass.BaseTestCase):
    def setUp(self):
        call_command('create_session', 'advance_slowest_wait', "2")
        self.session = Session.objects.get()



from otree.api import (
    Currency as c, currency_range, Submission, SubmissionMustFail)
from . import views
from ._builtin import Bot
from .models import Constants


class PlayerBot(Bot):

    def play_round(self):
        '''all args should autocomplete'''

        yield SubmissionMustFail(
            views.Page1,
            {'a': 1},
            check_html=True)

        yield Submission(
            views.Page1,
            {'a': 1},
            check_html=False,
            timeout_happened=True)



import os
import shutil
import tempfile

from django.core.management import call_command

from .utils import TestCase, capture_stdout, cd


class StartProjectTest(TestCase):
    def setUp(self):
        self.tmp_dir = tempfile.mkdtemp()
        print('tmp_dir:', self.tmp_dir)

    def tearDown(self):
        if os.path.exists(self.tmp_dir):
            shutil.rmtree(self.tmp_dir)

    def test_call_startproject(self):
        with cd(self.tmp_dir), capture_stdout():
            call_command('startproject', 'newproj', '--noinput')

        path = os.path.join(self.tmp_dir, 'newproj')
        self.assertTrue(os.path.exists(path))
        self.assertTrue(os.path.isdir(path))
        self.assertTrue(os.path.exists(os.path.join(path, 'settings.py')))



import os
from os import environ

import dj_database_url

import otree.settings


BASE_DIR = os.path.dirname(os.path.abspath(__file__))

# the environment variable OTREE_PRODUCTION controls whether Django runs in
# DEBUG mode. If OTREE_PRODUCTION==1, then DEBUG=False
if environ.get('OTREE_PRODUCTION') not in {None, '', '0'}:
    DEBUG = False
else:
    DEBUG = True

ADMIN_USERNAME = 'admin'

# for security, best to set admin password in an environment variable
ADMIN_PASSWORD = environ.get('OTREE_ADMIN_PASSWORD')

# don't share this with anybody.
SECRET_KEY = '{{ secret_key }}'

DATABASES = {
    'default': dj_database_url.config(
        # Rather than hardcoding the DB parameters here,
        # it's recommended to set the DATABASE_URL environment variable.
        # This will allow you to use SQLite locally, and postgres/mysql
        # on the server
        # Examples:
        # export DATABASE_URL=postgres://USER:PASSWORD@HOST:PORT/NAME
        # export DATABASE_URL=mysql://USER:PASSWORD@HOST:PORT/NAME

        # fall back to SQLite if the DATABASE_URL env var is missing
        default='sqlite:///' + os.path.join(BASE_DIR, 'db.sqlite3')
    )
}

# AUTH_LEVEL:
# If you are launching a study and want visitors to only be able to
# play your app if you provided them with a start link, set the
# environment variable OTREE_AUTH_LEVEL to STUDY.
# If you would like to put your site online in public demo mode where
# anybody can play a demo version of your game, set OTREE_AUTH_LEVEL
# to DEMO. This will allow people to play in demo mode, but not access
# the full admin interface.

AUTH_LEVEL = environ.get('OTREE_AUTH_LEVEL')

# setting for integration with AWS Mturk
AWS_ACCESS_KEY_ID = environ.get('AWS_ACCESS_KEY_ID')
AWS_SECRET_ACCESS_KEY = environ.get('AWS_SECRET_ACCESS_KEY')


# e.g. EUR, CAD, GBP, CHF, CNY, JPY
REAL_WORLD_CURRENCY_CODE = 'USD'
USE_POINTS = True


# e.g. en, de, fr, it, ja, zh-hans
# see: https://docs.djangoproject.com/en/1.9/topics/i18n/#term-language-code
LANGUAGE_CODE = 'en'

# if an app is included in SESSION_CONFIGS, you don't need to list it here
INSTALLED_APPS = ['otree']

# SENTRY_DSN = ''

DEMO_PAGE_INTRO_TEXT = """
oTree games
"""

mturk_hit_settings = {
    'keywords': ['easy', 'bonus', 'choice', 'study'],
    'title': 'Title for your experiment',
    'description': 'Description for your experiment',
    'frame_height': 500,
    'preview_template': 'global/MTurkPreview.html',
    'minutes_allotted_per_assignment': 60,
    'expiration_hours': 7*24,  # 7 days
    # 'grant_qualification_id': 'YOUR_QUALIFICATION_ID_HERE',# to prevent retakes
    # to use qualification requirements, you need to uncomment the 'qualification' import
    # at the top of this file.
    'qualification_requirements': [],
}

# if you set a property in SESSION_CONFIG_DEFAULTS, it will be inherited by all configs
# in SESSION_CONFIGS, except those that explicitly override it.
# the session config can be accessed from methods in your apps as self.session.config,
# e.g. self.session.config['participation_fee']

SESSION_CONFIG_DEFAULTS = {
    'real_world_currency_per_point': 0.000,
    'participation_fee': 0.00,
    'doc': "",
    'mturk_hit_settings': mturk_hit_settings,
}


SESSION_CONFIGS = [
    # {
    #     'name': '...',
    #     'display_name': '...',
    #     'num_demo_participants': ...,
    #     'app_sequence': ['...'],
    # }
]

# anything you put after the below line will override
# oTree's default settings. Use with caution.
otree.settings.augment_settings(globals())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import absolute_import

import json
import os
import random
import sys
from collections import OrderedDict

import channels
import vanilla
from django.conf import settings
from django.contrib import messages
from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect, JsonResponse, HttpResponse
from six.moves import range
from six.moves import zip

import otree.bots.browser
import otree.common_internal
import otree.export
from otree import forms
from otree.common import RealWorldCurrency
from otree.common_internal import (
    create_session_and_redirect, missing_db_tables,
    get_models_module, get_app_label_from_name, DebugTable
)
from otree.forms import widgets
from otree.management.cli import check_pypi_for_updates
from otree.models import Participant, Session
from otree.models_concrete import (
    ParticipantRoomVisit, BrowserBotsLauncherSessionCode)
from otree.room import ROOM_DICT
from otree.session import SESSION_CONFIGS_DICT, create_session, SessionConfig
from otree.views.abstract import GenericWaitPageMixin, AdminSessionPageMixin


def pretty_name(name):
    """Converts 'first_name' to 'first name'"""
    if not name:
        return ''
    return name.replace('_', ' ')


class CreateSessionForm(forms.Form):
    session_configs = SESSION_CONFIGS_DICT.values()
    session_config_choices = (
        [('', '-----')] +
        [(s['name'], s['display_name']) for s in session_configs])

    session_config = forms.ChoiceField(
        choices=session_config_choices, required=True)

    num_participants = forms.IntegerField()

    def __init__(self, *args, **kwargs):
        for_mturk = kwargs.pop('for_mturk')
        super(CreateSessionForm, self).__init__(*args, **kwargs)
        if for_mturk:
            self.fields['num_participants'].label = "Number of workers"
            self.fields['num_participants'].help_text = (
                'Since workers can return the hit or drop out '
                '"spare" participants will be created. Namely server will '
                'have %s times more participants than MTurk HIT. '
                'The number you enter in this field is number of '
                'workers required for your HIT.'
                % settings.MTURK_NUM_PARTICIPANTS_MULTIPLE
            )
        else:
            self.fields['num_participants'].label = "Number of participants"

    def clean_num_participants(self):
        session_config_name = self.cleaned_data.get('session_config')

        # We must check for an empty string in case validation is not run
        if session_config_name != '':
            lcm = SESSION_CONFIGS_DICT[session_config_name].get_lcm()
            num_participants = self.cleaned_data['num_participants']
            if num_participants % lcm:
                raise forms.ValidationError(
                    'Please enter a valid number of participants.'
                )
            return num_participants


class CreateSession(vanilla.FormView):
    form_class = CreateSessionForm
    template_name = 'otree/admin/CreateSession.html'

    url_pattern = r"^create_session/$"

    def dispatch(self, request, *args, **kwargs):
        self.for_mturk = (int(self.request.GET.get('mturk', 0)) == 1)
        return super(CreateSession, self).dispatch(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        kwargs['configs'] = SESSION_CONFIGS_DICT.values()
        return super(CreateSession, self).get_context_data(**kwargs)

    def get_form(self, data=None, files=None, **kwargs):
        kwargs['for_mturk'] = self.for_mturk
        return super(CreateSession, self).get_form(data, files, **kwargs)

    def form_valid(self, form):

        session_config_name = form.cleaned_data['session_config']
        session_kwargs = {
            'session_config_name': session_config_name,
            'for_mturk': self.for_mturk
        }
        if self.for_mturk:
            session_kwargs['num_participants'] = (
                form.cleaned_data['num_participants'] *
                settings.MTURK_NUM_PARTICIPANTS_MULTIPLE
            )

        else:
            session_kwargs['num_participants'] = (
                form.cleaned_data['num_participants'])

        # TODO:
        # Refactor when we upgrade to push
        if hasattr(self, "room"):
            session_kwargs['room_name'] = self.room.name

        post_data = self.request.POST
        config = SESSION_CONFIGS_DICT[session_config_name]

        edited_session_config_fields = {}

        for field in config.editable_fields():
            old_value = config[field]
            html_field_name = config.html_field_name(field)

            # int/float/decimal are set to required in HTML
            # bool:
            # - if unchecked, its key will be missing.
            # - if checked, its value will be 'on'.
            # str:
            # - if blank, its value will be ''
            new_value_str = post_data.get(html_field_name)
            # don't use isinstance because that will catch bool also
            if type(old_value) is int:
                # in case someone enters 1.0 instead of 1
                new_value = int(float(new_value_str))
            else:
                new_value = type(old_value)(new_value_str)
            if old_value != new_value:
                edited_session_config_fields[field] = new_value

        # need to convert to float or string in order to serialize
        # through channels
        for k in ['participation_fee', 'real_world_currency_per_point']:
            if k in edited_session_config_fields:
                edited_session_config_fields[k] = float(
                    edited_session_config_fields[k])
        session_kwargs['edited_session_config_fields'] = edited_session_config_fields
        return create_session_and_redirect(
            session_kwargs)


class Rooms(vanilla.TemplateView):
    template_name = 'otree/admin/Rooms.html'

    url_pattern = r"^rooms/$"

    def get_context_data(self, **kwargs):
        return {'rooms': ROOM_DICT.values()}


class RoomWithoutSession(CreateSession):
    template_name = 'otree/admin/RoomWithoutSession.html'
    room = None

    url_pattern = r"^room_without_session/(?P<room_name>.+)/$"

    def dispatch(self, request, *args, **kwargs):
        self.room = ROOM_DICT[kwargs['room_name']]
        if self.room.has_session():
            return HttpResponseRedirect(
                reverse('RoomWithSession', args=[kwargs['room_name']]))
        return super(RoomWithoutSession, self).dispatch(
            request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = {
            'participant_urls': self.room.get_participant_urls(self.request),
            'room_wide_url': self.room.get_room_wide_url(self.request),
            'room': self.room,
            'collapse_links': True,
        }
        kwargs.update(context)

        return super(RoomWithoutSession, self).get_context_data(**kwargs)

    def socket_url(self):
        return '/room_without_session/{}/'.format(self.room.name)


class RoomWithSession(vanilla.TemplateView):
    template_name = 'otree/admin/RoomWithSession.html'
    room = None

    url_pattern = r"^room_with_session/(?P<room_name>.+)/$"

    def dispatch(self, request, *args, **kwargs):
        self.room = ROOM_DICT[kwargs['room_name']]
        if not self.room.has_session():
            return HttpResponseRedirect(
                reverse('RoomWithoutSession', args=[kwargs['room_name']]))
        return super(RoomWithSession, self).dispatch(
            request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = {
            'participant_urls': self.room.get_participant_urls(self.request),
            'room_wide_url': self.room.get_room_wide_url(self.request),
            'session_url': reverse(
                'SessionMonitor',
                args=(self.room.session.code,)),
            'room': self.room,
            'collapse_links': True,
        }
        kwargs.update(context)

        return super(RoomWithSession, self).get_context_data(**kwargs)


class CloseRoom(vanilla.View):
    url_pattern = r"^CloseRoom/(?P<room_name>.+)/$"

    def post(self, request, *args, **kwargs):
        room_name = kwargs['room_name']
        self.room = ROOM_DICT[room_name]
        self.room.session = None
        # in case any failed to be cleared through regular ws.disconnect
        ParticipantRoomVisit.objects.filter(
            room_name=room_name,
        ).delete()
        return HttpResponseRedirect(
            reverse('RoomWithoutSession', args=[room_name]))


class WaitUntilSessionCreated(GenericWaitPageMixin, vanilla.GenericView):

    url_pattern = r"^WaitUntilSessionCreated/(?P<pre_create_id>.+)/$"

    title_text = 'Creating session'
    body_text = ''

    def _is_ready(self):
        try:
            self.session = Session.objects.get(
                _pre_create_id=self._pre_create_id
            )
            return True
        except Session.DoesNotExist:
            return False

    def _response_when_ready(self):
        session = self.session
        if session.is_for_mturk():
            session_home_url = reverse(
                'MTurkCreateHIT', args=(session.code,)
            )
        # demo mode
        elif self.request.GET.get('splitscreen'):
            session_home_url = reverse(
                'SessionSplitScreen', args=(session.code,))
        else:  # typical case
            session_home_url = reverse(
                'SessionStartLinks', args=(session.code,))

        return HttpResponseRedirect(session_home_url)

    def dispatch(self, request, *args, **kwargs):
        self._pre_create_id = kwargs['pre_create_id']
        if self._is_ready():
            return self._response_when_ready()
        return self._get_wait_page()

    def socket_url(self):
        return '/wait_for_session/{}/'.format(self._pre_create_id)


class SessionSplitScreen(AdminSessionPageMixin, vanilla.TemplateView):
    '''Launch the session in fullscreen mode
    only used in demo mode
    '''

    def get_context_data(self, **kwargs):
        '''Get the URLs for the IFrames'''
        context = super(SessionSplitScreen, self).get_context_data(**kwargs)
        participant_urls = [
            self.request.build_absolute_uri(participant._start_url())
            for participant in self.session.get_participants()
        ]
        context.update({
            'session': self.session,
            'participant_urls': participant_urls
        })
        return context


class SessionStartLinks(AdminSessionPageMixin, vanilla.TemplateView):

    def get_context_data(self, **kwargs):
        session = self.session
        room = session.get_room()

        context = super(SessionStartLinks, self).get_context_data(**kwargs)

        sqlite = settings.DATABASES['default']['ENGINE'].endswith('sqlite3')
        context.update({
            'use_browser_bots': session.use_browser_bots,
            'sqlite': sqlite,
            'runserver': 'runserver' in sys.argv
        })

        session_start_urls = [
            self.request.build_absolute_uri(participant._start_url())
            for participant in session.get_participants()
        ]

        # TODO: Bot URLs, and a button to start the bots

        if room:
            context.update(
                {
                    'participant_urls':
                        room.get_participant_urls(self.request),
                    'room_wide_url': room.get_room_wide_url(self.request),
                    'session_start_urls': session_start_urls,
                    'room': room,
                    'collapse_links': True,
                })
        else:
            anonymous_url = self.request.build_absolute_uri(
                reverse(
                    'JoinSessionAnonymously',
                    args=(session._anonymous_code,)
                )
            )

            context.update({
                'participant_urls': session_start_urls,
                'anonymous_url': anonymous_url,
                'num_participants': len(session_start_urls),
                'splitscreen_mode_on': len(session_start_urls) <= 3
            })

        return context


class SessionEditPropertiesForm(forms.ModelForm):
    participation_fee = forms.RealWorldCurrencyField(
        required=False,
        # it seems that if this is omitted, the step defaults to an integer,
        # meaninng fractional inputs are not accepted
        widget=widgets._RealWorldCurrencyInput(attrs={'step': 0.01})
    )
    real_world_currency_per_point = forms.FloatField(
        required=False
    )

    class Meta:
        model = Session
        fields = [
            'label',
            'experimenter_name',
            'comment',
        ]


class SessionEditProperties(AdminSessionPageMixin, vanilla.UpdateView):

    # required for vanilla.UpdateView
    lookup_field = 'code'
    model = Session
    form_class = SessionEditPropertiesForm
    template_name = 'otree/admin/SessionEditProperties.html'

    def get_form(self, data=None, files=None, **kwargs):
        form = super(
            SessionEditProperties, self
        ).get_form(data, files, **kwargs)
        config = self.session.config
        form.fields[
            'participation_fee'
        ].initial = config['participation_fee']
        form.fields[
            'real_world_currency_per_point'
        ].initial = config['real_world_currency_per_point']
        if self.session.mturk_HITId:
            form.fields['participation_fee'].widget.attrs['readonly'] = 'True'
        return form

    def get_success_url(self):
        return reverse('SessionEditProperties', args=(self.session.code,))

    def form_valid(self, form):
        super(SessionEditProperties, self).form_valid(form)
        participation_fee = form.cleaned_data[
            'participation_fee'
        ]
        real_world_currency_per_point = form.cleaned_data[
            'real_world_currency_per_point'
        ]
        config = self.session.config
        if form.cleaned_data['participation_fee'] is not None:
            config[
                'participation_fee'
            # need to convert back to RealWorldCurrency, because easymoney
            # MoneyFormField returns a decimal, not Money (not sure why)
            ] = RealWorldCurrency(participation_fee)
        if form.cleaned_data['real_world_currency_per_point'] is not None:
            config[
                'real_world_currency_per_point'
            ] = real_world_currency_per_point
        self.session.save()
        messages.success(self.request, 'Properties have been updated')
        return HttpResponseRedirect(self.get_success_url())


class SessionPayments(AdminSessionPageMixin, vanilla.TemplateView):

    def get(self, *args, **kwargs):
        response = super(SessionPayments, self).get(*args, **kwargs)
        return response

    def get_context_data(self, **kwargs):
        session = self.session
        # TODO: mark which ones are bots
        participants = session.get_participants()
        total_payments = 0.0
        mean_payment = 0.0
        if participants:
            total_payments = sum(
                part.payoff_plus_participation_fee() for part in participants
            )
            mean_payment = total_payments / len(participants)

        context = super(SessionPayments, self).get_context_data(**kwargs)
        context.update({
            'participants': participants,
            'total_payments': total_payments,
            'mean_payment': mean_payment,
            'participation_fee': session.config['participation_fee'],
        })

        return context


class SessionMonitor(AdminSessionPageMixin, vanilla.TemplateView):

    def get_context_data(self, **kwargs):

        field_names = otree.export.get_field_names_for_live_update(Participant)
        display_names = {
            '_id_in_session': 'ID in session',
            'code': 'Code',
            'label': 'Label',
            '_current_page': 'Page',
            '_current_app_name': 'App',
            '_round_number': 'Round',
            '_current_page_name': 'Page name',
            'status': 'Status',
            '_last_page_timestamp': 'Time on page',
        }

        column_names = [display_names[col] for col in field_names]

        context = super(SessionMonitor, self).get_context_data(**kwargs)
        context['column_names'] = column_names

        advance_users_button_text = (
            "Advance the slowest user(s) by one page, "
            "by forcing a timeout on their current page. "
        )
        if not self.session.pages_auto_reload_when_advanced():
            advance_users_button_text += (
                'NOTE: oTree is currently running in production mode, '
                'so participants will not see their page auto-advance until '
                'they refresh the page. This is a server performance '
                'optimization.'
            )
        context['advance_users_button_text'] = advance_users_button_text
        return context


def pretty_round_name(app_label, round_number):
    app_label = pretty_name(app_label)
    if round_number > 1:
        return '{} [Round {}]'.format(app_label, round_number)
    else:
        return app_label


class SessionData(AdminSessionPageMixin, vanilla.TemplateView):

    def get_context_data(self, **kwargs):
        session = self.session

        rows = []

        round_headers = []
        model_headers = []
        field_names = []

        # field names for JSON response
        field_names_json = []

        for subsession in session.get_subsessions():
            app_label = subsession._meta.app_config.name

            columns_for_models, subsession_rows = otree.export.get_rows_for_live_update(
                subsession._meta.app_config.name,
                subsession_pk=subsession.pk
            )

            if not rows:
                rows = subsession_rows
            else:
                for i in range(len(rows)):
                    rows[i].extend(subsession_rows[i])

            round_colspan = 0
            for model_name in ['player', 'group', 'subsession']:
                colspan = len(columns_for_models[model_name])
                model_headers.append((model_name.title(), colspan))
                round_colspan += colspan

            round_name = pretty_round_name(app_label, subsession.round_number)

            round_headers.append((round_name, round_colspan))

            this_round_fields = []
            this_round_fields_json = []
            for model_name in ['Player', 'Group', 'Subsession']:
                column_names = columns_for_models[model_name.lower()]
                this_model_fields = [pretty_name(n) for n in column_names]
                this_model_fields_json = [
                    '{}.{}.{}'.format(round_name, model_name, colname)
                    for colname in column_names
                ]
                this_round_fields.extend(this_model_fields)
                this_round_fields_json.extend(this_model_fields_json)

            field_names.extend(this_round_fields)
            field_names_json.extend(this_round_fields_json)

        # dictionary for json response
        # will be used only if json request  is done

        self.context_json = []
        for i, row in enumerate(rows, start=1):
            d_row = OrderedDict()
            # table always starts with participant 1
            d_row['participant_label'] = 'P{}'.format(i)
            for t, v in zip(field_names_json, row):
                d_row[t] = v
            self.context_json.append(d_row)

        context = super(SessionData, self).get_context_data(**kwargs)
        context.update({
            'subsession_headers': round_headers,
            'model_headers': model_headers,
            'field_headers': field_names,
            'rows': rows})
        return context

    def get(self, request, *args, **kwargs):
        context = self.get_context_data()
        if self.request.META.get('CONTENT_TYPE') == 'application/json':
            return JsonResponse(self.context_json, safe=False)
        else:
            return self.render_to_response(context)


class SessionDescription(AdminSessionPageMixin, vanilla.TemplateView):

    def get_context_data(self, **kwargs):
        context = super(SessionDescription, self).get_context_data(**kwargs)
        context['config'] = SessionConfig(self.session.config)
        return context


class AdminReportForm(forms.Form):
    app_name = forms.ChoiceField(choices=[], required=False)
    round_number = forms.IntegerField(required=False, min_value=1)

    def __init__(self, *args, **kwargs):
        self.session = kwargs.pop('session')
        super().__init__(*args, **kwargs)

        admin_report_apps = self.session._admin_report_apps()
        num_rounds_list = self.session._admin_report_num_rounds_list()
        self.rounds_per_app = dict(zip(admin_report_apps, num_rounds_list))
        app_name_choices = []
        for app_name in admin_report_apps:
            label = '{} ({} rounds)'.format(
                get_app_label_from_name(app_name), self.rounds_per_app[app_name]
            )
            app_name_choices.append((app_name, label))

        self.fields['app_name'].choices = app_name_choices

    def clean(self):
        cleaned_data = super().clean()

        apps_with_admin_report = self.session._admin_report_apps()

        # can't use setdefault because the key will always exist even if the
        # fields were empty.
        # str default value is '',
        # and int default value is None
        if not cleaned_data['app_name']:
            cleaned_data['app_name'] = apps_with_admin_report[0]

        rounds_in_this_app = self.rounds_per_app[cleaned_data['app_name']]

        round_number = cleaned_data['round_number']

        if not round_number or round_number > rounds_in_this_app:
            cleaned_data['round_number'] = rounds_in_this_app

        self.data = cleaned_data

        return cleaned_data

class AdminReport(AdminSessionPageMixin, vanilla.TemplateView):

    def get(self, request, *args, **kwargs):
        form = AdminReportForm(data=request.GET, session=self.session)
        # validate to get error messages
        form.is_valid()
        context = self.get_context_data(form=form)
        return self.render_to_response(context)

    def get_context_data(self, **kwargs):

        cleaned_data = kwargs['form'].cleaned_data

        models_module = get_models_module(cleaned_data['app_name'])
        subsession = models_module.Subsession.objects.get(
            session=self.session,
            round_number=cleaned_data['round_number'],
        )

        context = {
            'subsession': subsession,
            'Constants': models_module.Constants,
            'session': self.session,
            'user_template': '{}/AdminReport.html'.format(
            subsession._meta.app_config.label)
        }

        vars_for_admin_report = subsession.vars_for_admin_report() or {}
        self.debug_tables = [
            DebugTable(
                title='vars_for_admin_report',
                rows=vars_for_admin_report.items()
            )
        ]
        # determine whether to display debug tables
        self.is_debug = settings.DEBUG
        context.update(vars_for_admin_report)

        # this should take priority, in the event of a clash between
        # a user-defined var and a built-in one
        context.update(super().get_context_data(**kwargs))



        return context


class Sessions(vanilla.ListView):
    template_name = 'otree/admin/Sessions.html'

    url_pattern = r"^sessions/(?P<archive>archive)?$"

    def get_context_data(self, **kwargs):
        context = super(Sessions, self).get_context_data(**kwargs)
        context.update({
            'is_debug': settings.DEBUG,
        })
        return context

    def get_queryset(self):
        return Session.objects.filter(
            is_demo=False).order_by('archived', '-pk')


class ServerCheck(vanilla.TemplateView):
    template_name = 'otree/admin/ServerCheck.html'

    url_pattern = r"^server_check/$"

    def app_is_on_heroku(self):
        return 'heroku' in self.request.get_host()

    def timeoutworker_is_running(self):
        if otree.common_internal.USE_REDIS:
            redis_conn = otree.common_internal.get_redis_conn()
            return otree.bots.browser.ping_bool(redis_conn, timeout=2)
        else:
            # the timeoutworker relies on Redis (Huey),
            # so if Redis is not being used, the timeoutworker is not functional
            return False


    def get_context_data(self, **kwargs):
        sqlite = settings.DATABASES['default']['ENGINE'].endswith('sqlite3')
        debug = settings.DEBUG
        regular_sentry = hasattr(settings, 'RAVEN_CONFIG')
        heroku_sentry = os.environ.get('SENTRY_DSN')
        sentry = regular_sentry or heroku_sentry
        auth_level = settings.AUTH_LEVEL
        auth_level_ok = settings.AUTH_LEVEL in {'DEMO', 'STUDY'}
        heroku = self.app_is_on_heroku()
        runserver = 'runserver' in sys.argv
        db_synced = not missing_db_tables()
        pypi_results = check_pypi_for_updates()
        python2 = sys.version_info[0] == 2
        timeoutworker = self.timeoutworker_is_running()

        return {
            'sqlite': sqlite,
            'debug': debug,
            'sentry': sentry,
            'auth_level': auth_level,
            'auth_level_ok': auth_level_ok,
            'heroku': heroku,
            'runserver': runserver,
            'db_synced': db_synced,
            'pypi_results': pypi_results,
            'python2': python2,
            'timeoutworker': timeoutworker,
        }


class OtreeCoreUpdateCheck(vanilla.View):

    url_pattern = r"^version_cached/$"

    # cached per process
    results = None

    def get(self, request, *args, **kwargs):
        if OtreeCoreUpdateCheck.results is None:
            OtreeCoreUpdateCheck.results = check_pypi_for_updates()
        return JsonResponse(OtreeCoreUpdateCheck.results, safe=True)


class CreateBrowserBotsSession(vanilla.View):

    url_pattern = r"^create_browser_bots_session/$"

    def get(self, request, *args, **kwargs):
        # return browser bots check
        sqlite = settings.DATABASES['default']['ENGINE'].endswith('sqlite3')

        return JsonResponse({
            'sqlite': sqlite,
            'runserver': 'runserver' in sys.argv
        })

    def post(self, request, *args, **kwargs):
        num_participants = int(request.POST['num_participants'])
        session_config_name = request.POST['session_config_name']
        bot_case_number = int(request.POST['bot_case_number'])
        session = create_session(
            session_config_name=session_config_name,
            num_participants=num_participants,
            bot_case_number=bot_case_number,
            force_browser_bots=True
        )
        BrowserBotsLauncherSessionCode.objects.update_or_create(
            # i don't know why the update_or_create arg is called 'defaults'
            # because it will update even if the instance already exists
            # maybe for consistency with get_or_create
            defaults={'code': session.code}
        )
        channels.Group('browser_bot_wait').send(
            {'text': json.dumps({'status': 'session_ready'})}
        )

        return HttpResponse(session.code)


class CloseBrowserBotsSession(vanilla.View):

    url_pattern = r"^close_browser_bots_session/$"

    def post(self, request, *args, **kwargs):
        BrowserBotsLauncherSessionCode.objects.all().delete()
        return HttpResponse('ok')



from otree.api import Currency as c, currency_range
from . import models
from ._builtin import Page, WaitPage
from .models import Constants


class Wait1(WaitPage):
    # should autocomplete
    group_by_arrival_time = True

    def is_displayed(self):
        # player could be underlined in yellow,
        # because usually we don't use self.player in WaitPages
        # but could change in the future

        # return self.player.id_in_group == 1
        pass

    def get_players_for_group(self, waiting_players):
        pass


class Page1(Page):
    def vars_for_template(self):
        return {
            # should autocomplete
            'a': self.player.f_bool,
            'b': self.participant.vars.items(),
            'c': self.player.participant.session.config,
            'd': self.session.config,
        }

    # should autocomplete
    def get_timeout_seconds(self): pass

    def before_next_page(self):
        # should autocomplete
        _ = self.timeout_happened

page_sequence = [Wait1, Page1]



from . import models
from tests.utils import BlankTemplatePage
from otree.api import Currency as c

import sys, inspect


class Page(BlankTemplatePage):
    '''Most pages have just 1 field, so optimize for that but allow overriding'''

    form_model = models.Player
    yields = []

    def get_form_fields(self):
        return [self.field]

    @classmethod
    def get_yields(cls):
        # return a list of post dicts. only the last one should succeed
        return [{cls.field: x} for x in cls.yields]


class ChoicesFlat(Page):
    field = 'choices_flat'
    yields = [c(0), c(1)]


class Choices(Page):
    field = 'choices'
    yields = [0, 1]


class ErrorMessage(Page):
    def get_form_fields(self):
        return ['add100_1', 'add100_2']

    def error_message(self, values):
        if values['add100_1'] + values['add100_2'] != 100:
            return 'The numbers must add up to 100'

    @classmethod
    def get_yields(cls):
        return [
            {'add100_1': 1, 'add100_2': 98},
            {'add100_1': 1, 'add100_2': 99}
        ]


class FieldErrorMessage(Page):
    field = 'even_int'
    yields = [1, 2]

    def even_int_error_message(self, value):
        if value % 2:
            return 'Must be an even number'


class DynamicChoices(Page):
    field = 'dynamic_choices'
    yields = ['c', 'a']

    def dynamic_choices_choices(self):
        return [
            ['a', 'first choice'],
            ['b', 'second choice'],
        ]


class BlankDynamicChoices(Page):
    field = 'blank_dynamic_choices'

    @classmethod
    def get_yields(cls):
        return [{}]

    def blank_dynamic_choices_choices(self):
        return [1, 2]


class MinMax(Page):
    # test it on group also
    form_model = models.Group
    field = 'min_max'
    yields = [0, 2, 1]


class MinMaxDynamic(Page):
    field = 'min_max_dynamic'
    yields = [0, 2, 1]

    def min_max_dynamic_min(self):
        return 1

    def min_max_dynamic_max(self):
        # test referencing the instance
        return self.player.equals_one


class MinMaxBlank(Page):
    field = 'min_max_blank'

    @classmethod
    def get_yields(cls):
        # should succeed without any input
        return [{}]


class MinMaxBlank2(Page):
    field = 'min_max_blank2'
    yields = [2, 1]


def is_page_subclass(PageClass):
    return (
        inspect.isclass(PageClass)
        and issubclass(PageClass, Page)
        and PageClass != Page
    )


inspect_result = inspect.getmembers(sys.modules[__name__], is_page_subclass)

# getmembers returns  all the members of an object
# in a list of (name, value) pairs sorted by name.

page_sequence = [e[1] for e in inspect_result]



# -*- coding: utf-8 -*-
# <standard imports>
from __future__ import division
from otree.api import (
    BaseSubsession, BaseGroup, BasePlayer, BaseConstants, models
)
# </standard imports>

doc = ""

class Constants(BaseConstants):
    name_in_url = 'timeout'
    players_per_group = None
    num_rounds = 1
    invalid_f_char = 'invalid_string'


class Subsession(BaseSubsession):
    pass


class Group(BaseGroup):
    pass


class Player(BasePlayer):
    f_bool = models.BooleanField()
    f_posint = models.IntegerField(min=2)
    f_float = models.FloatField()
    f_currency = models.CurrencyField()
    f_char = models.CharField()

    foo = models.PositiveIntegerField()
    timeout_happened = models.BooleanField(initial=False)



# -*- coding: utf-8 -*-
from __future__ import division, absolute_import
from . import models
from otree.api import WaitPage, Page
from .models import Constants
from django.conf import settings


class Page1(Page):
    form_model = models.Player
    form_fields = ['contribution', 'yesno']
    timeout_seconds = 60


class MyWait(WaitPage):
    pass


page_sequence = [
    Page1,
    MyWait,
]



from . import views
from otree.api import Bot, SubmissionMustFail, Currency as c
import random



class PlayerBot(Bot):

    def play_round(self):
        for PageClass in views.page_sequence:
            yields = PageClass.get_yields()
            for submission in yields[:-1]:
                yield SubmissionMustFail(PageClass, submission)
            yield PageClass, yields[-1]


#!/usr/bin/env python
# -*- coding: utf-8 -*-


# =============================================================================
# IMPORTS
# =============================================================================

import logging

from django.conf import settings
from django.core.management.base import BaseCommand
from django.core.management import call_command
from django.db import connections, transaction
from django.db.migrations.loader import MigrationLoader
from django.db.migrations.autodetector import MigrationAutodetector

import six
from unittest import mock


# =============================================================================
# LOGGER
# =============================================================================

logger = logging.getLogger('otree')


def drop_tables_command(db_engine):
    if 'sqlite3' in db_engine:
        return 'DROP TABLE {table};'
    if 'oracle' in db_engine:
        return 'DROP TABLE "{table}" CASCADE CONSTRAINTS;'
    if 'postgres' in db_engine:
        return 'DROP TABLE "{table}" CASCADE;'
    if 'mysql' in db_engine:
        return (
            'SET FOREIGN_KEY_CHECKS = 0;'
            'DROP TABLE {table} CASCADE;'
            'SET FOREIGN_KEY_CHECKS = 1;')
    raise ValueError(
        'resetdb command does not recognize DB engine "{}"'.format(db_engine))


class Command(BaseCommand):
    help = (
        "Resets your development database to a fresh state. "
        "All data will be deleted.")

    def add_arguments(self, parser):
        ahelp = (
            'Tells the resetdb command to NOT prompt the user for '
            'input of any kind.')
        parser.add_argument(
            '--noinput', action='store_false', dest='interactive',
            default=True, help=ahelp)

    def _confirm(self):
        self.stdout.write(
            "This will delete and recreate your database. ")
        answer = six.moves.input("Proceed? (y or n): ")
        if answer:
            return answer[0].lower() == 'y'
        return False

    def _drop_table_stmt(self, dbconf):
        engine = dbconf["ENGINE"]
        return drop_tables_command(engine)

    def _get_tables(self, db):
        tables = []
        out = six.StringIO()
        call_command('inspectdb', database=db, no_color=True, stdout=out)
        for line in out.getvalue().splitlines():
            line = line.strip()
            if line.startswith("db_table = '"):
                tablename = line.replace(
                    "db_table = '", "", 1).replace("'", "").strip()
                tables.append(tablename)
        return tuple(reversed(tables))

    def _drop_tables(self, tables, db, dt_stmt):
        with connections[db].cursor() as cursor:
            for table in tables:
                stmt = dt_stmt.format(table=table)
                cursor.execute(stmt)

    def handle(self, **options):
        if options.pop("interactive") and not self._confirm():
            self.stdout.write('Canceled.')
            return

        for db, dbconf in six.iteritems(settings.DATABASES):
            db_engine = dbconf['ENGINE']
            if 'postgresql' in db_engine.lower():
                db_engine = 'PostgreSQL'
            elif 'sqlite' in db_engine.lower():
                db_engine = 'SQLite'
            elif 'mysql' in db_engine.lower():
                db_engine = 'MySQL'
            logger.info("Database engine: {}".format(db_engine))
            dt_stmt = self._drop_table_stmt(dbconf)

            logger.info("Retrieving Existing Tables...")
            tables = self._get_tables(db)

            logger.info("Dropping Tables...")

            # use a transaction to prevent the DB from getting in an erroneous
            # state, which can result in a different error message when resetdb
            # is run again, making the original error hard to trace.
            with transaction.atomic(
                    using=connections[db].alias,
                    savepoint=connections[db].features.can_rollback_ddl
            ):
                self._drop_tables(tables, db, dt_stmt)

                logger.info("Creating Database '{}'...".format(db))

                self.syncdb(db, options)

                # second call to 'migrate', simply to
                # fake migrations so that runserver doesn't complain
                # about unapplied migrations
                # note: In 1.9, will need to pass --run-syncdb flag

                call_command(
                    'migrate', database=db, fake=True,
                    interactive=False, **options)

        # mention the word 'columns' here, so people make the connection
        # between columns and resetdb, so that when they get a 'no such column'
        # error, they know how to fix it.
        # (An alternative is to generically catch "no such column" errors,
        # but I recall that this was difficult - because there were many
        # code paths or exception classes. Could re-investigate.)
        logger.info('Created new tables and columns.')

    @mock.patch.object(
        MigrationLoader, 'migrations_module',
        return_value='migrations nonexistent hack')
    @mock.patch.object(
        MigrationAutodetector, 'changes', return_value=False)
    def syncdb(self, db, options, *mocked_args):
        '''
        patch .migrations_module() to return a nonexistent module,
        instead of app_name.migrations.
        because this module is not found,
        migration system will assume the app has no migrations,
        and run syncdb instead.

        Hack so that migrate can't find migrations files
        this way, syncdb will be run instead of migrate.
        This is preferable because
        users who are used to running "otree resetdb"
        may not know how to run 'otree makemigrations'.
        This means their migration files will not be up to date,
        ergo migrate will create tables with an outdated schema.

        after the majority of oTree users have this new version
        of resetdb, we can add a migrations/ folder to each app
        in the sample games and the app template,
        and deprecate resetdb
        and instead use "otree makemigrations" and "otree migrate".

        also, syncdb is faster than migrate, and there is no
        advantage to migrate since it's being run on a newly
        created DB anyway.

        also patch MigrationAutodetector.changes() to suppress the warning
        "Your models have changes that are not yet reflected in a migration..."
        '''
        call_command(
            'migrate', database=db,
            interactive=False, **options)



import otree.session
from otree.bots.bot import ParticipantBot
from .utils import TestCase
from unittest import mock


class TestTemplates(TestCase):
    def test_all_blocks_found(self):
        session = otree.session.create_session(
            session_config_name='templates_app',
            num_participants=1,
            use_cli_bots=True,
            bot_case_number=0,
        )

        participant = session.get_participants()[0]
        bot = ParticipantBot(participant, load_player_bots=False)
        bot.open_start_url()
        for block_name in [
            'scripts', 'app_scripts', 'global_scripts',
            'styles', 'app_styles', 'global_styles',
        ]:
            self.assertIn('my_{}'.format(block_name), bot.html)



from django.core.urlresolvers import reverse
import splinter
from .utils import TestCase
from django.core.management import call_command
from otree.models.session import Session
from otree.models.participant import Participant


class TestSessionWideLink(TestCase):

    def setUp(self):
        self.browser = splinter.Browser('django') # type: splinter.Browser


    def test_open_participant_links(self):
        br = self.browser

        def go(url):
            br.visit(url)
            self.assertEqual(br.status_code, 200)

        call_command('create_session', 'simple', '3')
        session = Session.objects.get()

        without_label = reverse(
            'JoinSessionAnonymously', args=[session._anonymous_code])

        go(without_label)
        go(without_label)
        visited_count = Participant.objects.filter(visited=True).count()
        self.assertEqual(visited_count, 2)

        label = 'John'
        with_label = without_label + '?participant_label={}'.format(label)
        with_label2 = without_label + '?participant_label=John2'

        go(with_label)
        go(with_label)

        label_visited_count = Participant.objects.filter(
            visited=True, label=label).count()

        self.assertEqual(label_visited_count, 1)

        visited_count = Participant.objects.filter(
            visited=True).count()

        self.assertEqual(visited_count, 3)

        self.browser.visit(without_label)
        self.assertEqual(br.status_code, 404)

        self.browser.visit(with_label2)
        self.assertEqual(br.status_code, 404)



from otree.api import (
    models, BaseConstants, BaseSubsession, BaseGroup, BasePlayer,
    Currency as c, widgets
)


doc = """
Test general functionality:
-   Framework methods like creating_session, before_next_page, etc are executed
-   Modifications to objects are saved
"""


class Constants(BaseConstants):
    name_in_url = 'saving'
    players_per_group = None
    num_rounds = 1
    complex_data_structure = [
        {c(1): c(2)},
        {1.5: 2},
        {1,2}
    ]


class Subsession(BaseSubsession):

    def creating_session(self):
        self.session.vars['a'] = 1
        if self.round_number == 1:
            for p in self.get_players():
                p.participant.vars['a'] = 1
                p.participant.vars['c'] = Constants.complex_data_structure
            for g in self.get_groups():
                for p2 in g.get_players():
                    p2.participant.vars['b'] = 1
        for p3 in self.get_players():
            p3.in_creating_session = 1
        for g2 in self.get_groups():
            g2.in_creating_session = 1


class Group(BaseGroup):
    def set_payoffs(self):
        for p in self.get_players():
            p.payoff = c(50)

    in_creating_session = models.CurrencyField()


class Player(BasePlayer):

    after_next_button_field = models.BooleanField()
    in_creating_session = models.CurrencyField()



from django.core.management.commands import makemigrations
import six.moves

MIGRATIONS_WARNING = '''
oTree is not designed to be used with migrations. Instead, oTree uses the
'resetdb' command. You should only use 'makemigrations' and 'migrate'
if you understand how Django migrations work, and how to troubleshoot them.
'''


class Command(makemigrations.Command):

    def _confirm(self):
        self.stdout.write(MIGRATIONS_WARNING)
        answer = six.moves.input("Proceed? (y or n): ")
        if answer:
            return answer[0].lower() == 'y'
        return False

    def handle(self, *args, **options):

        if not self._confirm():
            self.stdout.write('Canceled.')
            return

        super(Command, self).handle(*args, **options)



from .utils import TestCase


class TestLegacy(TestCase):

    def test_boto2_import(self):
        '''testing the otree-boto2-shim'''
        from boto.mturk import qualification
        quals = [
            qualification.LocaleRequirement("EqualTo", "US"),
            qualification.PercentAssignmentsApprovedRequirement(
                     "GreaterThanOrEqualTo", 50),
            qualification.NumberHitsApprovedRequirement(
                     "GreaterThanOrEqualTo", 5)
        ]

        # make sure the import is actually using the boto2 shim,
        # not the actual boto2 package that may still be lying around
        for qual in quals:
            self.assertEqual(qual, None)



# -*- coding: utf-8 -*-
from __future__ import division, absolute_import
from . import models
from otree.api import WaitPage
from tests.utils import BlankTemplatePage as Page
from .models import Constants


class Page1(Page):
    form_model = models.Player
    form_fields = ['f_currency', 'f_bool', 'f_posint', 'f_char', 'f_float']

    timeout_seconds = 5

    def error_message(self, values):
        if values['f_char'] == Constants.invalid_f_char:
            return 'error!'

    def before_next_page(self):
        if self.timeout_happened:
            self.player.timeout_happened = True


class PageWithoutTimeout(Page):
    form_model = models.Player
    form_fields = ['foo']


page_sequence = [
    Page1,
    PageWithoutTimeout
]



import random
import decimal
import string

from django.utils import html
from django.template import Context, Template
from django.contrib.staticfiles.storage import staticfiles_storage

import six

from unittest import mock

from otree.common import Currency as c

from .utils import TestCase
from six.moves import range


class TestFilters(TestCase):

    def parse(self, fragment):
        return Template('{% load otree_tags %}' + fragment)

    def render(self, fragment, context=None):
        if context is None:
            context = Context()
        if not isinstance(context, Context):
            context = Context(context)
        return self.parse(fragment).render(context)

    def test_monkey_patch_staticfiles_tag(self):
        with mock.patch.object(staticfiles_storage, "url") as url:
            self.render("{% load staticfiles %}{% static 'foo.jpg' %}")
            url.assert_called_once_with("foo.jpg")
        with mock.patch.object(staticfiles_storage, "url") as url:
            url.side_effect = ValueError("boom")
            with self.assertRaises(ValueError):
                self.render("{% load staticfiles %}{% static 'foo.jpg' %}")

    def test_abs_value(self):
        for value in [0, 1, random.random(), c(1)]:
            actual = self.render("{{value|abs}}", context={'value': value})
            expected = six.text_type(value)
            self.assertEquals(actual, expected)

            nvalue = -value
            expected = self.render("{{value|abs}}", context={'value': nvalue})
            self.assertEquals(actual, expected)

        with self.assertRaises(TypeError):
            self.render("{{value|abs}}", context={'value': "foo"})
        with self.assertRaises(TypeError):
            self.render("{{value|abs}}", context={'value': None})

    def test_json(self):
        for python_input, expected_output in [
            (None, 'null'),
            (c(1), '1.0'),
            ("hi", '"hi"'),
            ({"a": 1}, '{"a": 1}')
        ]:
            output = self.render("{{value|json}}", context={'value': python_input})
            self.assertEquals(output, expected_output)


#!/usr/bin/env python
# encoding: utf-8

from django.views.generic import View
from django.http import JsonResponse
from django.http import Http404

from rest_framework import generics, permissions

from otree.models import Session
from otree.serializers import ParticipantSerializer


class Ping(View):

    def get(self, request):
        response = JsonResponse({"ping": True})
        response["Access-Control-Allow-Origin"] = "*"
        return response


class SessionParticipantsList(generics.ListCreateAPIView):
    serializer_class = ParticipantSerializer
    permission_classes = [permissions.AllowAny]

    def get_queryset(self):
        session_code = self.kwargs['session_code']
        try:
            session = Session.objects.get(code=session_code)
        except Session.DoesNotExist:
            raise Http404('This session no longer exists.')
        return session.participant_set.filter(visited=True)



from unittest import mock
from six import StringIO
from otree.management.commands.webandworkers import OTreeHonchoManager
from django.core.management import call_command
from otree.management import cli
from .utils import TestCase, capture_stdout


class OTreeAndDjangoVersion(TestCase):

    @mock.patch("otree.get_version", return_value="foo")
    @mock.patch("django.get_version", return_value="faa")
    def test_otree_and_django_version(self, dget_version, oget_version):
        actual = cli.otree_and_django_version()
        expected = 'oTree: foo - Django: faa'

        self.assertTrue(dget_version.called)
        self.assertTrue(oget_version.called)
        self.assertEqual(actual, expected)


class OTreeManagementUtility(TestCase):

    @mock.patch("sys.argv", new=["otree", "--help"])
    @mock.patch("platform.system", return_value="No-Windows")
    def test_help(self, *args):
        arguments = ["otree", "--help"]

        expected = StringIO()
        with mock.patch("sys.stdout", new=expected):
            cli.execute_from_command_line(arguments, "otree")

        utility = cli.OTreeManagementUtility(arguments)
        actual = StringIO()
        with mock.patch("sys.stdout", new=actual):
            utility.execute()

        self.assertEquals(actual.getvalue(), expected.getvalue())

    def test_commands_only(self, *args):
        utility = cli.OTreeManagementUtility([])
        main_help_text = utility.main_help_text().splitlines()
        for command in utility.main_help_text(commands_only=True).splitlines():
            prefix = "  {} - ".format(command)
            found = False
            for line in main_help_text:
                if line.startswith(prefix):
                    found = True
                    break
            if not found:
                self.fail("Command '{}' no found in help".format(command))

    def test_settings_exception(self):
        expected = ('Note that only Django core commands are listed as '
                    'settings are not properly configured (error: foo).')

        utility = cli.OTreeManagementUtility([])
        utility.settings_exception = "foo"
        main_help_text = utility.main_help_text().splitlines()

        self.assertEquals(main_help_text[-1], expected)

        utility = cli.OTreeManagementUtility([])
        utility.settings_exception = None
        main_help_text = utility.main_help_text().splitlines()

        self.assertNotEquals(main_help_text[-1], expected)


class ExecuteFromCommandLine(TestCase):

    @mock.patch("platform.system", return_value="No-Windows")
    @mock.patch("otree.management.cli.OTreeManagementUtility")
    def test_execute_from_command_line_runserver(self, *args):
        management, system = args
        cli.execute_from_command_line(["otree", "runserver"], "script.py")
        management.assert_called_with(["otree", "runserver"])

    # not working at the moment because of SSL server
    '''
    @mock.patch("platform.system", return_value="No-Windows")
    @mock.patch("otree.management.cli.OTreeManagementUtility")
    @mock.patch("django.conf.LazySettings.AWS_ACCESS_KEY_ID", create=True)
    def test_execute_from_command_line_runserver_ssh(self, *args):
        key, management, system = args
        cli.execute_from_command_line(["otree", "runserver"], "script.py")
        management.assert_called_with(["otree", "runsslserver"])
    '''

    # not working at the moment because of pypi cli check
    '''
    @mock.patch("platform.system", return_value="No-Windows")
    @mock.patch("sys.stdout")
    @mock.patch("otree.management.cli.otree_and_django_version",
                return_value="foo")
    def test_execute_from_command_line_runserver_no_env_command(self, *args):
        version, stdout, system = args
        cli.execute_from_command_line(["version"], "script.py")
        self.assertTrue(version.called)
        stdout.write.assert_called_with("foo\n")
    '''

from collections import namedtuple
CommandAndResult = namedtuple(
    'CommandAddrPort', field_names=['command', 'address', 'port'])


class OTreeCli(TestCase):

    @mock.patch("sys.argv", new=["--help"])
    @mock.patch("otree.management.cli.execute_from_command_line")
    def test_clean_run(self, execute_from_command_line):
        cli.otree_cli()
        execute_from_command_line.assert_called_with(["--help"], 'otree')

    @mock.patch("sys.argv", new=["--version"])
    @mock.patch("otree.management.cli.execute_from_command_line")
    @mock.patch("os.getcwd", return_value="foo")
    def test_add_pwd(self, *args):
        gcwd, execute_from_command_line = args
        with mock.patch("sys.path", new=[]) as path:
            cli.otree_cli()
            self.assertEquals(path, ["foo"])
        self.assertTrue(gcwd.called)
        execute_from_command_line.assert_called_with(["--version"], 'otree')

    # 2017-06-17:
    # what's the point of patching execute_from_command_line and calling otree_cli?
    # why not just use call_command? That seems better because it covers calls
    # using 'otree' script and 'python manage.py', and just tests the command itself,
    # not that argv is passed correctly, which Django handles for us
    # also it's more standard

    @mock.patch("sys.exit")
    @mock.patch.dict('os.environ', {'PORT': '5000'})
    @mock.patch.object(OTreeHonchoManager, 'loop')
    @mock.patch.object(OTreeHonchoManager, 'add_otree_process')
    def test_webandworkers(self, add_otree_process, *args):
        specs = [
            # should respect the $PORT env var, for Heroku
            ('webandworkers', '0.0.0.0', '5000'),
            # new syntax, to be consistent with runserver
            ('runprodserver 127.0.0.1:80', '127.0.0.1', '80'),
            ('runprodserver 8002', '0.0.0.0', '8002'),
            # legacy syntax
            ('runprodserver --port=81', '0.0.0.0', '81')
        ]

        for command, expected_address, expected_port in specs:
            add_otree_process.reset_mock()
            with capture_stdout():
                call_command(*command.split(' '))
            add_otree_process.assert_any_call(
                'daphne',
                'daphne otree.asgi:channel_layer -b {} -p {}'.format(
                    expected_address, expected_port))

    @mock.patch("sys.exit")
    @mock.patch.object(OTreeHonchoManager, 'loop')
    @mock.patch.object(OTreeHonchoManager, 'add_otree_process')
    def test_runprodserver(self, add_otree_process, *args):
        with capture_stdout():
            call_command('runprodserver')
        add_otree_process.assert_any_call(
            'botworker', 'otree botworker')
        add_otree_process.assert_any_call(
            'timeoutworkeronly',
            'otree timeoutworkeronly',
        )



import os
import sys
import tempfile

from .utils import TestCase, add_path, capture_stdout, cd


class AddPathTest(TestCase):
    def test_add_path(self):
        tmpdir = tempfile.mkdtemp()
        original_path = sys.path[:]
        with add_path(tmpdir):
            self.assertEqual(len(sys.path), len(original_path) + 1)
            self.assertTrue(tmpdir in sys.path)
        self.assertEqual(sys.path, original_path)
        self.assertTrue(tmpdir not in sys.path)


class CaptureStdoutTest(TestCase):
    def test_capture(self):
        with capture_stdout() as output:
            print("Hello world!")
        self.assertEqual(output.read(), "Hello world!\n")

    def test_stdout_chagned(self):
        original_stdout = sys.stdout
        self.assertTrue(sys.stdout is original_stdout)
        with capture_stdout():
            self.assertFalse(sys.stdout is original_stdout)
        self.assertTrue(sys.stdout is original_stdout)


class CdTest(TestCase):
    def test_cd(self):
        tmpdir = tempfile.mkdtemp()
        # OS X returns symlinks for the temporary directory.
        # We need to resolve to the real path in order to reliably test the
        # outcome of a os.chdir()
        # See: http://stackoverflow.com/q/12482702/199848
        tmpdir = os.path.realpath(tmpdir)

        current_cwd = os.getcwd()
        self.assertEqual(os.getcwd(), current_cwd)
        with cd(tmpdir):
            self.assertEqual(os.getcwd(), tmpdir)
        self.assertEqual(os.getcwd(), current_cwd)

    def test_cd_with_cwd(self):
        current_cwd = os.getcwd()
        self.assertEqual(os.getcwd(), current_cwd)
        with cd(current_cwd):
            self.assertEqual(os.getcwd(), current_cwd)
        self.assertEqual(os.getcwd(), current_cwd)



from . import views
from .models import Constants
from otree.api import Bot
from unittest import TestCase


class PlayerBot(Bot):

    def play_round(self):
        # add these in later
        assert self.group.set_in_wait_page == True, self.group.id
        assert self.player.set_in_wait_page == True

        yield views.Page1

        self.run_assertions()

    def run_assertions(self):
        if self.round_number == Constants.num_rounds:
            for p in self.player.in_all_rounds():
                assert p.group.player_set.count() == Constants.players_per_group



# -*- coding: utf-8 -*-
from __future__ import division, absolute_import
from . import views
from .models import Constants
from otree.api import Bot

class GBATMixin:
    def assert_same_group_members_in_all_rounds(self):
        if self.player.id_in_group == 1 and self.subsession.round_number == Constants.num_rounds:
            id_lists = []
            for group in self.group.in_all_rounds():
                participant_ids = list(group.player_set.values_list(
                    'participant_id', flat=True))
                id_lists.append(participant_ids)
            assert [id_list == id_lists[0] for id_list in id_lists]

    def assert_no_empty_groups(self):
        num_players = self.subsession.player_set.count()
        if self.player.id_in_subsession == num_players:
            num_groups_without_players = type(self.group).objects.filter(player__isnull=True).count()
            assert num_groups_without_players == 0

    def assert_same_id_in_group_in_all_rounds(self):
        if self.round_number == Constants.num_rounds:
            ids_in_group = [p.id_in_group for p in self.player.in_all_rounds()]
            assert all(id_in_group == ids_in_group[0] for id_in_group in ids_in_group)

    def run_assertions(self):
        self.assert_same_group_members_in_all_rounds()
        self.assert_no_empty_groups()
        self.assert_same_id_in_group_in_all_rounds()

class PlayerBot(Bot, GBATMixin):

    def play_round(self):
        assert len(self.group.get_players()) == 2
        yield views.Page1

        self.run_assertions()




from otree.api import (
    BaseSubsession, BaseGroup, BasePlayer, BaseConstants, models,
    Currency
)

doc = """
Testing cases
"""

class Constants(BaseConstants):
    name_in_url = 'cases'
    players_per_group = None
    num_rounds = 1


class Subsession(BaseSubsession):
    pass


class Group(BaseGroup):
    pass


class Player(BasePlayer):
    pass



import importlib
import sys

import django.test.client
import splinter
import splinter.browser
from django.conf import settings
from django.core.urlresolvers import reverse

from .utils import OTreePhantomBrowser, TestCase, get_path


def reload_urlconf():
    if settings.ROOT_URLCONF in sys.modules:
        module = sys.modules[settings.ROOT_URLCONF]
    else:
        module = importlib.import_module(settings.ROOT_URLCONF)
    importlib.reload(module)


class TestAdminBasic(TestCase):

    def setUp(self):
        self.browser = django.test.client.Client()

    def test_admin_basic(self):
        for tab in [
            'demo',
            'sessions',
            'rooms',
            'create_session',
            'server_check',
            'accounts/login'
        ]:
            response = self.browser.get('/{}/'.format(tab), follow=True)
            self.assertEqual(response.status_code, 200)

    def test_login(self):
        login_url = '/accounts/login/'
        resp = self.browser.post(
            login_url,
            data={
                'username': settings.ADMIN_USERNAME,
                'password': settings.ADMIN_PASSWORD,
            },
            follow=True
        )

        self.assertEqual(resp.status_code, 200)
        self.assertNotIn(login_url, get_path(resp, if_no_redirect=login_url))

    def test_auth_level(self):

        # whether you should be granted access when not logged in,
        # based on AUTH_LEVEL
        auth_level_outcomes = {
            'STUDY': {
                'OutOfRangeNotification': True,
                'DemoIndex': False,
                'ExportIndex': False,
            },
            'DEMO': {
                'OutOfRangeNotification': True,
                'DemoIndex': True,
                'ExportIndex': False,
            },
            '': {
                'OutOfRangeNotification': True,
                'DemoIndex': True,
                'ExportIndex': True,
            }
        }

        for AUTH_LEVEL in auth_level_outcomes:
            with self.settings(AUTH_LEVEL=AUTH_LEVEL):
                reload_urlconf()
                for url_name in auth_level_outcomes[AUTH_LEVEL]:
                    url = reverse(url_name)
                    resp = self.browser.get(url, follow=True)
                    is_login_page = any(
                        ['accounts/login' in ele[0]
                         for ele in resp.redirect_chain])
                    access_granted = not is_login_page
                    self.assertEqual(
                        access_granted,
                        auth_level_outcomes[AUTH_LEVEL][url_name],
                        msg='AUTH_LEVEL={}, url_name={}, access_granted={}'.format(
                            AUTH_LEVEL, url_name, access_granted
                        )
                    )
            # reload URLconf to restore back to its original state,
            # so other tests can run normally
            reload_urlconf()

'''
# We can't enable these test cases until we upgrade to CHannels 1.0+
class TestAdminJS(ChannelLiveServerTestCase):

    def setUp(self):
        self.browser = splinter.Browser('phantomjs')

    def test_create_session(self):
        br = self.browser
        create_session_url = '{}{}'.format(self.live_server_url, reverse('CreateSession'))
        br.visit(create_session_url)
        br.fill_form({
            'session_config': 'simple',
            'num_participants': '1',
        })
        button = br.find_by_value('Create')
        button.click()
        self.assertTrue(br.is_text_present('Simple Game: session', wait_time=3))


class TestRoomJS(ChannelLiveServerTestCase):

    def test_presence(self):
        # admin browser
        abr = OTreePhantomBrowser(live_server_url=self.live_server_url)

        # participant browser
        pbr = OTreePhantomBrowser(live_server_url=self.live_server_url)

        room_name = 'with_labels'

        # participant opens waiting page
        room_url = reverse('AssignVisitorToRoom', args=[room_name])
        pbr.go(room_url)

        pbr.fill('participant_label', 'JohnSmith')
        button = pbr.find_by_tag('button')
        button.click()
        waiting_url = pbr.url

        # admin opens RoomWithoutSession
        abr.go(reverse('RoomWithoutSession', args=[room_name]))

        # within a few seconds, he should see that participant online,
        # but not another participant.
        self.assertTrue(
            abr.is_element_present_by_css(
                '#present-participant-label-JohnSmith.count-this', wait_time=5))
        self.assertFalse(
            abr.is_element_present_by_css(
                '#present-participant-label-Bob.count-this'))

        # if participant closes the browser, he should go offline
        # go to some other URL
        pbr.go('/')

        self.assertFalse(
            abr.is_element_present_by_css(
                '#present-participant-label-JohnSmith.count-this', wait_time=1))

        # comes back online
        # need to use visit, not go, because it's an absolute URL
        pbr.visit(waiting_url)

        # admin creates a session in the room
        abr.fill_form({
            'session_config': 'simple',
            'num_participants': '1',
        })
        button = abr.find_by_value('Create')
        button.click()

        # within a few seconds, the participant should be redirected
        my_field_present = pbr.is_element_present_by_name('my_field', wait_time=2)
        self.assertTrue(my_field_present)
'''


#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import absolute_import

import json
import threading
import logging
from collections import OrderedDict

from django.test import SimpleTestCase

import channels
import traceback

import otree.common_internal
from otree.models.participant import Participant
from otree.common_internal import get_redis_conn

from .bot import ParticipantBot


REDIS_KEY_PREFIX = 'otree-bots'

# if you are testing all configs from the CLI browser bot launcher,
# and each app has multiple cases, it's possible to end up with many
# bots in the history.
# usually this wouldn't matter,
# but timeoutworker may try to load the pages after they have been completed
# (it will POST then get redirected to GET)
SESSIONS_PRUNE_LIMIT = 80

# global variable that holds the browser bot worker instance in memory
browser_bot_worker = None  # type: Worker

# these locks are only necessary when using runserver
# because then the botworker stuff is done by one of the 4 worker threads.
prepare_submit_lock = threading.Lock()
add_or_remove_bot_lock = threading.Lock()

logger = logging.getLogger('otree.test.browser_bots')


class Worker(object):
    def __init__(self, redis_conn=None):
        self.redis_conn = redis_conn
        self.participants_by_session = OrderedDict()
        self.browser_bots = {}
        self.prepared_submits = {}

    def initialize_participant(self, participant_code):
        self.prune()

        participant = Participant.objects.select_related('session').get(
            code=participant_code)
        session_code = participant.session.code

        # in order to do .assertEqual etc, need to pass a reference to a
        # SimpleTestCase down to the Player bot
        test_case = SimpleTestCase()

        self.browser_bots[participant.code] = ParticipantBot(
            participant, unittest_case=test_case)

        with add_or_remove_bot_lock:
            if session_code not in self.participants_by_session:
                self.participants_by_session[session_code] = []
            self.participants_by_session[session_code].append(participant_code)

        return {'ok': True}

    def get_method(self, command_name):
        commands = {
            'prepare_next_submit': self.prepare_next_submit,
            'consume_next_submit': self.consume_next_submit,
            'initialize_participant': self.initialize_participant,
            'clear_all': self.clear_all,
            'ping': self.ping,
        }

        return commands[command_name]

    def prune(self):
        '''to avoid memory leaks'''
        with add_or_remove_bot_lock:
            if len(self.participants_by_session) > SESSIONS_PRUNE_LIMIT:
                _, p_codes = self.participants_by_session.popitem(last=False)
                for participant_code in p_codes:
                    self.browser_bots.pop(participant_code, None)

    def clear_all(self):
        self.browser_bots.clear()

    def consume_next_submit(self, participant_code):
        submission = self.prepared_submits.pop(participant_code)
        # maybe was popped in prepare_next_submit
        submission.pop('page_class', None)
        return submission

    def prepare_next_submit(self, participant_code, path, html):

        try:
            bot = self.browser_bots[participant_code]
        except KeyError:
            return {
                'request_error': (
                    "Participant {} not loaded in botworker. "
                    "This can happen for several reasons: "
                    "(1) You are running multiple botworkers "
                    "(2) You restarted the botworker after creating the session "
                    "(3) The bots expired "
                    "(the botworker stores bots for "
                    "only the most recent {} sessions).".format(
                        participant_code, SESSIONS_PRUNE_LIMIT)
                )
            }

        # so that any asserts in the PlayerBot work.
        bot.path = path
        bot.html = html

        with prepare_submit_lock:
            if participant_code in self.prepared_submits:
                return {}

            try:
                submission = next(bot.submits_generator)
            except StopIteration:
                # don't prune it because can cause flakiness if
                # there are other GET requests coming in. it will be pruned
                # when new sessions are created anyway.

                # need to return something, to distinguish from Redis timeout
                submission = {}
            else:
                # because we are returning it through Redis, need to pop it
                # here
                submission.pop('page_class')

            self.prepared_submits[participant_code] = submission

        return submission

    def ping(self, *args, **kwargs):
        return {'ok': True}

    def redis_listen(self):
        print('botworker is listening for messages through Redis')
        while True:
            retval = None

            # blpop returns a tuple
            result = None

            # put it in a loop so that we can still receive KeyboardInterrupts
            # otherwise it will block
            while result is None:
                result = self.redis_conn.blpop(REDIS_KEY_PREFIX, timeout=3)

            key, message_bytes = result
            message = json.loads(message_bytes.decode('utf-8'))
            response_key = message['response_key']

            try:
                cmd = message['command']
                args = message.get('args', [])
                kwargs = message.get('kwargs', {})
                method = self.get_method(cmd)
                retval = method(*args, **kwargs)
            except Exception as exc:
                # request_error means the request received through Redis
                # was invalid.
                # response_error means the botworker raised while processing
                # the request.
                retval = {
                    'response_error': repr(exc),
                    'traceback': traceback.format_exc()
                }
                # don't raise, because then this would crash.
                # logger.exception() will record the full traceback
                logger.exception(repr(exc))
            finally:
                retval_json = json.dumps(retval or {})
                self.redis_conn.rpush(response_key, retval_json)


class BotWorkerPingError(Exception):
    pass


def ping(redis_conn, *, timeout):
    '''
    timeout arg is required because this is often called together
    with another function that has a timeout. need to be aware of double
    timeouts piling up.
    '''
    unique_response_code = otree.common_internal.random_chars_8()
    response_key = '{}-ping-{}'.format(REDIS_KEY_PREFIX, unique_response_code)
    msg = {
        'command': 'ping',
        'response_key': response_key,
    }
    redis_conn.rpush(REDIS_KEY_PREFIX, json.dumps(msg))
    # make it very long, so we don't get spurious ping errors
    result = redis_conn.blpop(response_key, timeout)

    if result is None:
        raise BotWorkerPingError(
            'Ping to botworker failed. '
            'If you want to use browser bots, '
            'you need to be running the botworker '
            '(which is started automatically if you run "otree runprodserver" '
            'or "otree timeoutworker"). '
            'Otherwise, set ("use_browser_bots": False) in the session config '
            'in settings.py.'
        )


def ping_bool(redis_conn, *, timeout):
    '''version of ping that returns True/False rather than raising'''
    try:
        ping(redis_conn, timeout=timeout)
        return True
    except BotWorkerPingError:
        return False


def load_redis_response_dict(response_bytes):
    response = json.loads(response_bytes.decode('utf-8'))
    # response_error only exists if using Redis.
    # if using runserver, there is no need for this because the
    # exception is raised in the same thread.
    if 'traceback' in response:
        # cram the other traceback in this traceback message.
        # note:
        raise Exception(response['traceback'])
    return response


def initialize_bot_redis(redis_conn, participant_code):
    response_key = '{}-initialize-{}'.format(REDIS_KEY_PREFIX, participant_code)
    msg = {
        'command': 'initialize_participant',
        'kwargs': {'participant_code': participant_code},
        'response_key': response_key,
    }
    # ping will raise if it times out
    ping(redis_conn, timeout=5)
    redis_conn.rpush(REDIS_KEY_PREFIX, json.dumps(msg))

    # timeout must be int
    # this is about 20x as much time as it should take
    # some users were still getting timeout errors with timeout=1
    timeout = 2
    result = redis_conn.blpop(response_key, timeout=timeout)
    if result is None:
        raise Exception(
            'botworker is running but could not initialize the bot '
            'within {} seconds.'.format(timeout)
        )
    key, response_bytes = result
    load_redis_response_dict(response_bytes)
    return {'ok': True}


def initialize_bot_in_process(participant_code):
    browser_bot_worker.initialize_participant(participant_code)


def initialize_bot(participant_code):
    if otree.common_internal.USE_REDIS:
        initialize_bot_redis(
            redis_conn=get_redis_conn(),
            participant_code=participant_code,
        )
    else:
        initialize_bot_in_process(participant_code)


def redis_flush_bots(redis_conn):
    for key in redis_conn.scan_iter(match='{}*'.format(REDIS_KEY_PREFIX)):
        redis_conn.delete(key)


class EphemeralBrowserBot(object):

    def __init__(self, view, redis_conn=None):
        self.view = view
        self.participant = view.participant
        self.session = self.view.session
        self.redis_conn = redis_conn or get_redis_conn()
        self.path = self.view.request.path

    def prepare_next_submit_redis(self, html):
        participant_code = self.participant.code
        response_key = '{}-prepare_next_submit-{}'.format(
            REDIS_KEY_PREFIX, participant_code)
        msg = {
            'command': 'prepare_next_submit',
            'kwargs': {
                'participant_code': participant_code,
                'path': self.path,
                'html': html,
            },
            'response_key': response_key,
        }
        return self.get_redis_response(msg)

    def get_redis_response(self, msg: dict):
        response_key = msg['response_key']
        redis_conn = self.redis_conn
        redis_conn.rpush(REDIS_KEY_PREFIX, json.dumps(msg))
        # in practice is very fast...around 1ms
        # however, if an exception occurs, could take quite long.
        # so, make this very long so we don't get spurious errors.
        # no advantage to cutting it off early.
        # if it's that slow consistently, people will complain.
        result = redis_conn.blpop(response_key, timeout=6)
        if result is None:
            # ping will raise if it times out
            ping(redis_conn, timeout=3)
            raise Exception(
                'botworker is running but did not return a submission.'
            )
        key, submit_bytes = result
        return load_redis_response_dict(submit_bytes)

    def prepare_next_submit_in_process(self, html):
        return browser_bot_worker.prepare_next_submit(
            self.participant.code, self.path, html)

    def prepare_next_submit(self, html):
        if otree.common_internal.USE_REDIS:
            result = self.prepare_next_submit_redis(html)
        else:
            result = self.prepare_next_submit_in_process(html)
        if 'request_error' in result:
            raise AssertionError(result['request_error'])

    def get_next_post_data_redis(self):
        participant_code = self.participant.code
        redis_conn = self.redis_conn
        response_key = '{}-consume_next_submit-{}'.format(
            REDIS_KEY_PREFIX, participant_code)
        msg = {
            'command': 'consume_next_submit',
            'kwargs': {
                'participant_code': participant_code,
            },
            'response_key': response_key,
        }
        return self.get_redis_response(msg)

    def get_next_post_data(self):
        if otree.common_internal.USE_REDIS:
            submission = self.get_next_post_data_redis()
        else:
            submission = browser_bot_worker.prepared_submits.pop(
                self.participant.code)
        if submission:
            return submission['post_data']
        else:
            raise StopIteration('No more submits')

    def send_completion_message(self):
        channels.Group(
            'browser-bots-client-{}'.format(self.session.code)
        ).send({'text': self.participant.code})



from otree.api import (
    models, BaseConstants, BaseSubsession, BaseGroup, BasePlayer,
    Currency as c, widgets
)


doc = """
Test misc functionality of a 1-player game
"""


class Constants(BaseConstants):
    name_in_url = 'form_validation'
    players_per_group = None
    num_rounds = 1


class Subsession(BaseSubsession):
    pass

class Group(BaseGroup):

    # example field
    min_max = models.CurrencyField(
        doc="""
        Description of this field, for documentation
        """,
        min=1,
        max=1
    )

    min_max_dynamic = models.CurrencyField()


class Player(BasePlayer):

    blank = models.CharField(blank=True)

    add100_1 = models.PositiveIntegerField()
    add100_2 = models.PositiveIntegerField()

    even_int = models.PositiveIntegerField()

    dynamic_choices = models.CharField()

    blank_dynamic_choices = models.IntegerField(blank=True)

    choices_flat = models.CurrencyField(
        widget=widgets.RadioSelect(),
        choices=[c(1), c(2)]
    )

    choices = models.IntegerField(
        widget=widgets.RadioSelect(),
        choices=[[1, 'A'], [2, 'B']]
    )

    dynamic_radio = models.CharField(widget=widgets.RadioSelectHorizontal())

    min_max_dynamic = models.CurrencyField()
    min_max_blank = models.FloatField(blank=True, min=1, max=1)
    min_max_blank2 = models.FloatField(blank=True, min=1, max=1)

    equals_one = models.IntegerField(initial=1)


#!/usr/bin/env python
# encoding: utf-8

import warnings
import datetime
from collections import defaultdict
import sys
import logging

from django.conf import settings
from django.contrib import messages
from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect, HttpResponseServerError
from django.shortcuts import get_object_or_404

from six.moves.urllib.parse import urlparse
from six.moves.urllib.parse import urlunparse

import vanilla

try:
    import boto3
except ImportError:
    boto3 = None

import IPy

import otree
from otree import forms
from otree.deprecate import OtreeDeprecationWarning
from otree.views.abstract import AdminSessionPageMixin
from otree.checks.mturk import MTurkValidator
from otree.forms import widgets
from otree.common import RealWorldCurrency
from otree.models import Session
from decimal import Decimal

logger = logging.getLogger('otree')

import contextlib


def get_mturk_client(*, use_sandbox=True):

    if use_sandbox:
        endpoint_url = 'https://mturk-requester-sandbox.us-east-1.amazonaws.com'
    else:
        endpoint_url = 'https://mturk-requester.us-east-1.amazonaws.com'
    return boto3.client(
        'mturk',
        aws_access_key_id=settings.AWS_ACCESS_KEY_ID,
        aws_secret_access_key=settings.AWS_SECRET_ACCESS_KEY,
        endpoint_url=endpoint_url,
        # if I specify endpoint_url without region_name, it complains
        region_name='us-east-1',
    )


@contextlib.contextmanager
def MTurkClient(*, use_sandbox=True, request):
    '''Alternative to get_mturk_client, for when we need exception handling
    in admin views, we should pass it, so that we can show the user the message
    without crashing.
    for participant-facing views and commandline tools, should use get_mturk_client.
    '''
    try:
        yield get_mturk_client(use_sandbox=use_sandbox)
    except Exception as exc:
        logger.error('MTurk error', exc_info=True)
        messages.error(request, str(exc), extra_tags='safe')


def get_all_assignments(mturk_client, hit_id):
    # Accumulate all relevant assignments, one page of results at
    # a time.
    assignments = []

    args = dict(
        HITId=hit_id,
        # i think 100 is the max page size
        MaxResults=100,
        AssignmentStatuses=['Submitted', 'Approved', 'Rejected']
    )

    while True:
        response = mturk_client.list_assignments_for_hit(**args)
        if not response['Assignments']:
            break
        assignments.extend(response['Assignments'])
        args['NextToken'] = response['NextToken']

    return assignments


def get_workers_by_status(mturk_client, hit_id):
    all_assignments = get_all_assignments(mturk_client, hit_id)
    workers_by_status = defaultdict(list)
    for assignment in all_assignments:
        workers_by_status[
            assignment['AssignmentStatus']
        ].append(assignment['WorkerId'])
    return workers_by_status



class MTurkCreateHITForm(forms.Form):

    use_sandbox = forms.BooleanField(
        required=False,
        label='Use MTurk Sandbox (for development and testing)',
        help_text=(
            "If this box is checked, your HIT will not be published to "
            "the MTurk live site, but rather to the MTurk Sandbox, "
            "so you can test how it will look to MTurk workers."
        ))
    title = forms.CharField()
    description = forms.CharField()
    keywords = forms.CharField()
    money_reward = forms.RealWorldCurrencyField(
        # it seems that if this is omitted, the step defaults to an integer,
        # meaninng fractional inputs are not accepted
        widget=widgets._RealWorldCurrencyInput(attrs={'step': 0.01})
    )
    assignments = forms.IntegerField(
        label="Number of assignments",
        help_text="How many unique Workers do you want to work on the HIT?")
    minutes_allotted_per_assignment = forms.IntegerField(
        label="Minutes allotted per assignment",
        help_text=(
            "Number of minutes, that a Worker has to "
            "complete the HIT after accepting it."
        ))
    expiration_hours = forms.FloatField(
        label="Hours until HIT expiration",
        help_text=(
            "Number of hours after which the HIT "
            "is no longer available for users to accept. "
        ))

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields['assignments'].widget.attrs['readonly'] = True


class MTurkCreateHIT(AdminSessionPageMixin, vanilla.FormView):
    '''This view creates mturk HIT for session provided in request
    AWS externalQuestion API is used to generate HIT.

    '''
    form_class = MTurkCreateHITForm

    def in_public_domain(self, request, *args, **kwargs):
        """This method validates if oTree are published on a public domain
        because mturk need it

        """
        host = request.get_host().lower()
        if ":" in host:
            host = host.split(":", 1)[0]
        if host == "localhost":
            return False
        try:
            ip = IPy.IP(host)
            return ip.iptype() == "PUBLIC"
        except ValueError:
            # probably is a public domain
            return True

    def get(self, request, *args, **kwargs):

        mturk_settings = self.session.config['mturk_hit_settings']


        initial = {
            'title': mturk_settings['title'],
            'description': mturk_settings['description'],
            'keywords': ', '.join(mturk_settings['keywords']),
            'money_reward': self.session.config['participation_fee'],
            'use_sandbox': settings.DEBUG,
            'minutes_allotted_per_assignment': (
                mturk_settings['minutes_allotted_per_assignment']
            ),
            'expiration_hours': mturk_settings['expiration_hours'],
            'assignments': self.session.mturk_num_participants,
        }

        form = self.get_form(initial=initial)
        context = self.get_context_data(form=form)

        url = self.request.build_absolute_uri(
            reverse('MTurkCreateHIT', args=(self.session.code,))
        )
        parsed_url = urlparse(url)
        https = parsed_url.scheme == 'https'
        secured_url = urlunparse(parsed_url._replace(scheme='https'))

        aws_keys_exist = bool(settings.AWS_ACCESS_KEY_ID) and bool(settings.AWS_SECRET_ACCESS_KEY)
        boto3_installed = bool(boto3)
        mturk_ready = aws_keys_exist and boto3_installed and https
        missing_next_button_warning = MTurkValidator(self.session).validation_message()

        context.update({
            # boto3 module must be imported, not None
            'boto3_installed': boto3_installed,
            'https': https,
            'aws_keys_exist': aws_keys_exist,
            'mturk_ready': mturk_ready,
            'runserver': 'runserver' in sys.argv,
            'secured_url': secured_url,
            'missing_next_button_warning': missing_next_button_warning
        })

        return self.render_to_response(context)

    def post(self, request, *args, **kwargs):
        form = self.get_form(
            data=request.POST,
            files=request.FILES
        )
        if not form.is_valid():
            return self.form_invalid(form)
        session = self.session
        use_sandbox = 'use_sandbox' in form.data
        # session can't be created
        if (not self.in_public_domain(request, *args, **kwargs) and
           not use_sandbox):
                msg = (
                    '<h1>Error: '
                    'oTree must run on a public domain for Mechanical Turk'
                    '</h1>')
                return HttpResponseServerError(msg)
        mturk_settings = session.config['mturk_hit_settings']
        qualification_id = mturk_settings.get(
            'grant_qualification_id', None)
        # verify that specified qualification type
        # for preventing retakes exists on mturk server

        url_landing_page = self.request.build_absolute_uri(
            reverse('MTurkLandingPage', args=(session.code,)))

        # updating schema from http to https
        # this is compulsory for MTurk exteranlQuestion
        # TODO: validate, that the server support https
        #       (heroku does support by default)
        secured_url_landing_page = urlunparse(
            urlparse(url_landing_page)._replace(scheme='https'))

        # TODO: validate that there is enough money for the hit
        money_reward = form.cleaned_data['money_reward']

        # assign back to participation_fee, in case it was changed
        # in the form
        # need to convert back to RealWorldCurrency, because easymoney
        # MoneyFormField returns a decimal, not Money (not sure why)
        # see views.admin.EditSessionProperties
        session.config['participation_fee'] = RealWorldCurrency(money_reward)

        external_question = '''
        <ExternalQuestion xmlns="http://mechanicalturk.amazonaws.com/AWSMechanicalTurkDataSchemas/2006-07-14/ExternalQuestion.xsd">
          <ExternalURL>{}</ExternalURL>
          <FrameHeight>{}</FrameHeight>
        </ExternalQuestion>
        '''.format(secured_url_landing_page, mturk_settings['frame_height'])

        qualifications = mturk_settings.get('qualification_requirements')

        if qualifications and not isinstance(qualifications[0], dict):
            raise ValueError(
                'settings.py: You need to upgrade your MTurk qualification_requirements '
                'to the boto3 format. See the documentation.'
            )

        mturk_hit_parameters = {
            'Title': form.cleaned_data['title'],
            'Description': form.cleaned_data['description'],
            'Keywords': form.cleaned_data['keywords'],
            'Question': external_question,
            'MaxAssignments': form.cleaned_data['assignments'],
            'Reward': str(float(money_reward)),
            'QualificationRequirements': qualifications,
            'AssignmentDurationInSeconds': 60*form.cleaned_data['minutes_allotted_per_assignment'],
            'LifetimeInSeconds': int(60*60*form.cleaned_data['expiration_hours']),
            # prevent duplicate HITs
            'UniqueRequestToken':'otree_{}'.format(session.code),
        }

        with MTurkClient(use_sandbox=use_sandbox, request=request) as mturk_client:
            if qualification_id:
                try:
                    mturk_client.get_qualification_type(
                        QualificationTypeId=qualification_id)
                # it's RequestError, but
                except Exception as exc:
                    if use_sandbox:
                        sandbox_note = (
                            'You are currently using the sandbox, so you '
                            'can only grant qualifications that were '
                            'also created in the sandbox.')
                    else:
                        sandbox_note = (
                            'You are using the MTurk live site, so you '
                            'can only grant qualifications that were '
                            'also created on the live site, and not the '
                            'MTurk sandbox.')
                    msg = (
                        "In settings.py you specified qualification ID '{qualification_id}' "
                        "MTurk returned the following error: [{exc}] "
                        "Note: {sandbox_note}".format(
                            qualification_id=qualification_id,
                            exc=exc,
                            sandbox_note=sandbox_note))
                    messages.error(request, msg)
                    return HttpResponseRedirect(
                        reverse(
                            'MTurkCreateHIT', args=(session.code,)))

            hit = mturk_client.create_hit(**mturk_hit_parameters)['HIT']

            session.mturk_HITId = hit['HITId']
            session.mturk_HITGroupId = hit['HITGroupId']
            session.mturk_use_sandbox = use_sandbox
            session.save()

        return HttpResponseRedirect(
            reverse('MTurkCreateHIT', args=(session.code,)))


class MTurkSessionPayments(AdminSessionPageMixin, vanilla.TemplateView):

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        session = self.session
        if not session.mturk_HITId:
            context.update({'not_published_yet': True})
            return context
        with MTurkClient(use_sandbox=session.mturk_use_sandbox, request=self.request) as mturk_client:
            workers_by_status = get_workers_by_status(
                mturk_client,
                session.mturk_HITId
            )
            participants_not_reviewed = session.participant_set.filter(
                mturk_worker_id__in=workers_by_status['Submitted']
            )
            participants_approved = session.participant_set.filter(
                mturk_worker_id__in=workers_by_status['Approved']
            )
            participants_rejected = session.participant_set.filter(
                mturk_worker_id__in=workers_by_status['Rejected']
            )

            context.update({
                'participants_approved': participants_approved,
                'participants_rejected': participants_rejected,
                'participants_not_reviewed': participants_not_reviewed,
                'participation_fee': session.config['participation_fee'],
            })

        return context


class PayMTurk(vanilla.View):

    url_pattern = r'^PayMTurk/(?P<session_code>[a-z0-9]+)/$'

    def post(self, request, *args, **kwargs):
        session = get_object_or_404(otree.models.Session,
                                    code=kwargs['session_code'])
        successful_payments = 0
        failed_payments = 0
        mturk_client = get_mturk_client(use_sandbox=session.mturk_use_sandbox)
        # use worker ID instead of assignment ID. Because 2 workers can have
        # the same assignment (if 1 starts it then returns it). we can't really
        # block that.
        # however, we can ensure that 1 worker does not get 2 assignments,
        # by enforcing that the same worker is always assigned to the same participant.
        for p in session.participant_set.filter(
            mturk_worker_id__in=request.POST.getlist('workers')
        ):
            # need the try/except so that we try to pay the rest of the participants
            payoff = p.payoff_in_real_world_currency()

            try:
                # approve assignment
                mturk_client.approve_assignment(AssignmentId=p.mturk_assignment_id)
                if payoff > 0:
                    mturk_client.send_bonus(
                        WorkerId=p.mturk_worker_id,
                        AssignmentId=p.mturk_assignment_id,
                        BonusAmount='{0:.2f}'.format(Decimal(payoff)),
                        # prevent duplicate payments
                        UniqueRequestToken='{}_{}'.format(p.mturk_worker_id, p.mturk_assignment_id),
                        # although the Boto documentation doesn't say so,
                        # this field is required. A user reported:
                        # "Value null at 'reason' failed to satisfy constraint:
                        # Member must not be null."
                        Reason='Thank you'
                    )
                successful_payments += 1
            except Exception as e:
                msg = (
                    'Could not pay {} because of an error communicating '
                    'with MTurk: {}'.format(p._id_in_session(), str(e)))
                messages.error(request, msg)
                logger.error(msg)
                failed_payments += 1
        msg = 'Successfully made {} payments.'.format(successful_payments)
        if failed_payments > 0:
            msg += ' {} payments failed.'.format(failed_payments)
            messages.warning(request, msg)
        else:
            messages.success(request, msg)
        return HttpResponseRedirect(
            reverse('MTurkSessionPayments', args=(session.code,)))


class RejectMTurk(vanilla.View):

    url_pattern = r'^RejectMTurk/(?P<session_code>[a-z0-9]+)/$'

    def post(self, request, *args, **kwargs):
        session = get_object_or_404(Session,
                                    code=kwargs['session_code'])
        with MTurkClient(use_sandbox=session.mturk_use_sandbox,
                         request=request) as mturk_client:

            for p in session.participant_set.filter(
                mturk_worker_id__in=request.POST.getlist('workers')
            ):
                mturk_client.reject_assignment(
                    AssignmentId=p.mturk_assignment_id,
                    # The boto3 docs say this param is optional, but if I omit it, I get:
                    # An error occurred (ValidationException) when calling the RejectAssignment operation:
                    # 1 validation error detected: Value null at 'requesterFeedback'
                    # failed to satisfy constraint: Member must not be null
                    RequesterFeedback=''
                )

            messages.success(request, "You successfully rejected "
                                      "selected assignments")
        return HttpResponseRedirect(
            reverse('MTurkSessionPayments', args=(session.code,)))




from otree.api import Bot, Submission
from . import views
from .models import Constants


class PlayerBot(Bot):

    def play_round(self):
        id_in_group = self.player.id_in_group

        if id_in_group == 1:
            yield views.HtmlMissingFields, {'field_not_in_template': 0.1}
        elif id_in_group == 2:
            yield views.HtmlMissingButton
        else:
            yield Submission(views.TimeoutPage, timeout_happened=True)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging

import six

from django.core.management.base import BaseCommand

from otree.session import create_session
from otree.room import ROOM_DICT


# =============================================================================
# LOGGING
# =============================================================================

logger = logging.getLogger('otree')


# =============================================================================
# COMMAND
# =============================================================================

class Command(BaseCommand):
    help = "oTree: Create a session."

    def add_arguments(self, parser):
        parser.add_argument(
            'session_config_name', type=six.u, help="The session config name")
        parser.add_argument(
            'num_participants', type=int,
            help="Number of participants for the created session")
        parser.add_argument(
            "-l", "--label", action="store", type=six.u,
            dest="label", default='', help="label for the created session")
        parser.add_argument(
            "--room", action="store", type=six.u,
            dest="room", default=None,
            help="Name of room to create the session in")

    def handle(self, *args, **options):
        session_config_name = options["session_config_name"]
        num_participants = options["num_participants"]
        label = options['label']

        session = create_session(
            session_config_name=session_config_name,
            num_participants=num_participants, label=label)

        if options['room']:
            room = ROOM_DICT[options['room']]
            room.session = session
            logger.info("Created session with code {} in room '{}'\n".format(
                session.code, options['room']))
        else:
            logger.info("Created session with code {}\n".format(session.code))
